# -*- coding: utf-8 -*-
# Copyright (C) 2012, Almar Klein
#
# Visvis is distributed under the terms of the (new) BSD License.
# The full license can be found in 'license.txt'.
import visvis as vv

def use(backendName=None):
    """ use(backendName=None)
    
    Use the specified backend and return an App instance that has a Run()
    method to enter the GUI toolkit's mainloop, and a ProcessEvents()
    method to process any GUI events.
    
    This function is only required to explicitly choose a specific backend,
    or to obtain the application object; when this function is not used,
    vv.figure() will select a backend automatically.
    
    Backend selection
    -----------------
    If no backend is given, returns the previously selected backend. If no
    backend was yet selected, a suitable backend is selected automatically.
    This is done by detecting whether any of the backend toolkits is
    already loaded. If not, visvis tries to load the
    vv.settings.preferredBackend first.
    
    Note: the backend can be changed even when figures are created with
    another backend, but this is not recommended.
    
    Example embedding in Qt4
    ------------------------
    # Near the end of the script:
    
    # Get app instance and create native app
    app = vv.use('pyside')
    app.Create()
    
    # Create window
    m = MainWindow()
    m.resize(560, 420)
    m.show()
    
    # Run main loop
    app.Run()
    
    """
    return vv.backends.use(backendName)

if __name__=='__main__':
    app = vv.use() # No arg provided: select backend automatically
    print('Selected backend is %s.' % app.GetBackendName())
    app.Create() # Create the backend's application
    app.Run() # Enter the backend's mainloop (not in interactive mode)



# -*- coding: utf-8 -*-
# Copyright (C) 2012, Almar Klein
#
# Visvis is distributed under the terms of the (new) BSD License.
# The full license can be found in 'license.txt'.

import visvis as vv
import numpy as np


class BarDescription:
    def __init__(self, sze):
        self.i = int(np.random.uniform(0,sze))
        self.j = int(np.random.uniform(0,sze))
        self.k1 = int(np.random.uniform(0,sze/3))
        self.k2 = int(np.random.uniform(0,sze/3))
        self.w1 = int(np.random.uniform(1,10))
        self.w2 = int(np.random.uniform(1,10))
        self.value = np.random.uniform(0,1)


def aVolume(N=5, size=64):
    """ aVolume(N=5, size=64)
    
    Creates a volume (3D image) with random bars.
    The returned numpy array has values between 0 and 1.
    Intended for quick illustration and test purposes.
    
    Parameters
    ----------
    N : int
        The number of bars for each dimension.
    size : int
        The size of the volume (for each dimension).
    
    """
    
    # Create volume
    vol = np.zeros((size,size,size), dtype=np.float32)

    # Make bars
    for iter in range(N):
        # x
        b = BarDescription(size)
        vol[ b.i-b.w1:b.i+b.w1, b.j-b.w2:b.j+b.w2, b.k1:-b.k2 ] += b.value
        # y
        b = BarDescription(size)
        vol[ b.i-b.w1:b.i+b.w1, b.k1:-b.k2, b.j-b.w2:b.j+b.w2 ] += b.value
        # z
        b = BarDescription(size)
        vol[ b.k1:-b.k2, b.i-b.w1:b.i+b.w1, b.j-b.w2:b.j+b.w2 ] += b.value

    # Clip and return
    vol[vol>1.0]=1.0
    return vol


if __name__ == '__main__':
    vol = vv.aVolume()
    t = vv.volshow(vol)
    t.renderStyle = 'mip' # maximum intensity projection (is the default)



# -*- coding: utf-8 -*-
# Copyright (C) 2012, Almar Klein
#
# Visvis is distributed under the terms of the (new) BSD License.
# The full license can be found in 'license.txt'.

# Modified for wxPython Pheonix 3.0.0.0
# Keith Smith 15 Oct 2013

""" The WX backend.


"""

# NOTICE: wx has the same general problem with OpenGl being kinda
# unmanaged and frames not being drawn on Gnome. However, wx seems
# relatively well workable with only applying a Refresh command
# on each Activate command of the main window.

import os

import visvis
from visvis import BaseFigure, events, constants
from visvis.core.misc import getResourceDir

import wx
from wx.glcanvas import GLCanvas, GLContext


KEYMAP = {  wx.WXK_SHIFT: constants.KEY_SHIFT,
            wx.WXK_ALT: constants.KEY_ALT,
            wx.WXK_CONTROL: constants.KEY_CONTROL,
            wx.WXK_LEFT: constants.KEY_LEFT,
            wx.WXK_UP: constants.KEY_UP,
            wx.WXK_RIGHT: constants.KEY_RIGHT,
            wx.WXK_DOWN: constants.KEY_DOWN,
            wx.WXK_PAGEUP: constants.KEY_PAGEUP,
            wx.WXK_PAGEDOWN: constants.KEY_PAGEDOWN,
            wx.WXK_RETURN: constants.KEY_ENTER,
            wx.WXK_ESCAPE: constants.KEY_ESCAPE,
            wx.WXK_DELETE: constants.KEY_DELETE
            }

# Make uppercase letters be lowercase
for i in range(ord('A'), ord('Z')):
    KEYMAP[i] = i+32


def modifiers(event):
    """Convert the WX modifier state into a tuple of active modifier keys."""
    mod = ()
    if event.ShiftDown():
        mod += constants.KEY_SHIFT,
    if event.CmdDown():
        mod += constants.KEY_CONTROL,
    if event.AltDown():
        mod += constants.KEY_ALT,
    return mod

#===============================================================================
# This is the Widget class prior to wxPython Phoenix
#===============================================================================
class GLWidget(GLCanvas):
    """ Implementation of the WX GLCanvas, which passes a number of
    events to the Figure object that wraps it.
    
    This is the original version with automatic glContext switching
    """
    
    def __init__(self, figure, parent, *args, **kwargs):
        # make sure the window is double buffered (Thanks David!)
        kwargs.update({'attribList' : [wx.glcanvas.WX_GL_RGBA,
            wx.glcanvas.WX_GL_DOUBLEBUFFER]})
        # call GLCanvas' init method
        GLCanvas.__init__(self, parent, *args, **kwargs)
        self._glContext = GLContext(self )
        self._glContext.SetCurrent(self)
        self.figure = figure
        
        # find root window
        root = self.GetParent()
        while root.GetParent():
            root = root.GetParent()
        
        # make bindings for events
        self.Bind(wx.EVT_LEFT_DOWN, self.OnLeftDown)
        self.Bind(wx.EVT_LEFT_UP, self.OnLeftUp)
        self.Bind(wx.EVT_RIGHT_DOWN, self.OnRightDown)
        self.Bind(wx.EVT_RIGHT_UP, self.OnRightUp)
        self.Bind(wx.EVT_LEFT_DCLICK, self.OnDoubleClick)
        self.Bind(wx.EVT_RIGHT_DCLICK, self.OnDoubleClick)
        self.Bind(wx.EVT_MOUSEWHEEL, self.OnMouseWheel)
        #
        self.Bind(wx.EVT_MOTION, self.OnMotion)
        self.Bind(wx.EVT_ENTER_WINDOW, self.OnEnter)
        self.Bind(wx.EVT_LEAVE_WINDOW, self.OnLeave)
        self.Bind(wx.EVT_KEY_DOWN, self.OnKeyDown)
        self.Bind(wx.EVT_KEY_UP, self.OnKeyUp)
        self.Bind(wx.EVT_SIZE, self.OnResize)
        root.Bind(wx.EVT_CLOSE, self.OnClose) # Note root
        #
        self.Bind(wx.EVT_PAINT, self.OnPaint)
        self.Bind(wx.EVT_SET_FOCUS, self.OnFocus)
        self.Bind(wx.EVT_ERASE_BACKGROUND, self.OnEraseBackground)
        root.Bind(wx.EVT_ACTIVATE, self.OnActivate) # Note root
        
        # Needs to focus to catch key events
        self.SetFocus()
        
        # if lost, tough luck (thus the comment)
        #self.Bind(wx.EVT_MOUSE_CAPTURE_LOST, self.OnMouseUp)
        
        # onpaint is called when shown is called by figure() function.
    
    
    def GetContext(self):
        return self._glContext

    def OnLeftDown(self, event):
        x,y = event.GetPosition()
        self.CaptureMouse() # make sure to capture release outside
        self.figure._GenerateMouseEvent('down', x, y, 1, modifiers(event))

    def OnLeftUp(self, event):
        x,y = event.GetPosition()
        try:
            self.ReleaseMouse()
        except Exception:
            pass
        self.figure._GenerateMouseEvent('up', x, y, 1, modifiers(event))

    def OnRightDown(self, event):
        x,y = event.GetPosition()
        self.CaptureMouse() # make sure to capture release outside
        self.figure._GenerateMouseEvent('down', x, y, 2, modifiers(event))

    def OnRightUp(self, event):
        x,y = event.GetPosition()
        try:
            self.ReleaseMouse()
        except Exception:
            pass
        self.figure._GenerateMouseEvent('up', x, y, 2, modifiers(event))
    
    def OnDoubleClick(self, event):
        but = 0
        x,y = event.GetPosition()
        if event.LeftDClick():
            but = 1
        elif event.RightDClick():
            but = 2
        self.figure._GenerateMouseEvent('double', x, y, but, modifiers(event))
    
    def OnMotion(self, event):
        if self.figure:
            # poduce event
            x,y = event.GetPosition()
            self.figure._GenerateMouseEvent('motion', x, y, 0, modifiers(event))
    
    def OnMouseWheel(self, event):
        numDegrees = event.GetWheelRotation() / 8.0
        numSteps = numDegrees / 15.0
        # There is event.GetWheelAxis() but only in newer versions of wx
        # Mine has not so I am not able to even test it...
        horizontal, vertical = 0, numSteps
        if self.figure:
            x,y = event.GetPosition()
            self.figure._GenerateMouseEvent('scroll', x, y, horizontal, vertical, modifiers(event))
    
    def OnKeyDown(self, event):
        key, text = self._ProcessKey(event)
        self.figure._GenerateKeyEvent('keydown', key, text, modifiers(event))
    
    def OnKeyUp(self, event):
        key, text = self._ProcessKey(event)
        self.figure._GenerateKeyEvent('keyup', key, text, modifiers(event))
    
    
    def _ProcessKey(self,event):
        """ evaluates the keycode of wx, and transform to visvis key.
        Also produce text version.
        return key, text. """
        key = event.GetKeyCode()
        # special cases for shift control and alt -> map to 17 18 19
        if key in KEYMAP:
            return KEYMAP[key], ''
        else:
            # other key, try producing text
            if (65 <= key <= 90) and not event.ShiftDown():
                key += 32
            try:
                return key, chr(key)
            except ValueError:
                return key, ''
    
    def OnEnter(self, event):
        if self.figure:
            ev = self.figure.eventEnter
            ev.Set(0,0,0)
            ev.Fire()
    
    def OnLeave(self, event):
        if self.figure:
            ev = self.figure.eventLeave
            ev.Set(0,0,0)
            ev.Fire()
        
    def OnResize(self, event):
        if self.figure:
            self.figure._OnResize()
            event.Skip()
    
    def OnClose(self, event):
        if self.figure:
            self.figure.Destroy()
            parent = self.GetParent()
            self.Destroy() # Hide and delete window
            # Prevent frame from sticking when there is no wx event loop
            if isinstance(parent, FigureFrame):
                parent.Hide()
        event.Skip()
    
    def OnFocus(self, event):
        if self.figure:
            BaseFigure._currentNr = self.figure.nr
            event.Skip()

    def OnPaint(self, event):
        # I read that you should always create a PaintDC when implementing
        # an OnPaint event handler.
        a = wx.PaintDC(self)  # noqa
        if self.GetContext():
            self.figure.OnDraw()
        event.Skip()
    
    def OnActivate(self, event):
        # When the title bar is dragged in ubuntu
        if event.GetActive():
            self.Refresh()
        event.Skip()
    
    def OnEraseBackground(self, event):
        pass # This prevents flicker on Windows


#===============================================================================
# This is the new widget class for wxPython Phoenix
#===============================================================================
class GLWidgetPhoenix(GLWidget):
    """ Implementation of the WX GLCanvas, which passes a number of
    events to the Figure object that wraps it.
    
    This new Phoenix version must explicitly set the glContext
    in OnFocus and OnPaint
    """
    
    def OnFocus(self, event):
        if self.figure and self.IsShownOnScreen():
            # Must set glContext for Phoenix
            self.SetCurrent(self._glContext)
            BaseFigure._currentNr = self.figure.nr
            event.Skip()

    def OnPaint(self, event):
        # I read that you should always create a PaintDC when implementing
        # an OnPaint event handler.
        # a = wx.PaintDC(self)
        
        # Must set glContext for Phoenix, but only if  shown on screen
        if self.GetContext() and self.IsShownOnScreen():
            
            self.SetCurrent(self._glContext)
            self.figure.OnDraw()
        event.Skip()


class Figure(BaseFigure):
    """ This is the wxPython implementation of the figure class.
    
    A Figure represents the OpenGl context and is the root
    of the visualization tree; a Figure Wibject does not have a parent.
    
    A Figure can be created with the function vv.figure() or vv.gcf().
    """
    
    def __init__(self, parent, *args, **kwargs):
        
        self._widget = None
        self._widget_args = (parent, args, kwargs)
        if kwargs.get('create_widget', True):
            self.CreateWidget()
        
        # call original init AFTER we created the widget
        BaseFigure.__init__(self)
    
    def CreateWidget(self):
        """ Create the Figure's widget if necessary, and return the
        widget. """
        if self._widget is None:
            # Make sure there is a native app and the timer is started
            # (also when embedded)
            app.Create()
            
            # create widget
            updatePosition = False
            parent, args, kwargs = self._widget_args
            if 'create_widget' in kwargs:
                updatePosition = True
                del(kwargs['create_widget'])
                
            # Based on switch set in App the correct widget class is called
            if app._phoenix:
                self._widget = GLWidgetPhoenix(self, parent, *args, **kwargs)
            else:
                self._widget = GLWidget(self, parent, *args, **kwargs)

            if updatePosition:
                self.position._Changed()
        return self._widget
    
    
    def _SetCurrent(self):
        """ make this scene the current context """
        if not self._destroyed and self._widget is not None:
            try:
                self._widget.SetCurrent()
            except Exception:
                # can happen when trying to call this method after
                # the window was destroyed.
                pass
    
    
    def _SwapBuffers(self):
        """ Swap the memory and screen buffer such that
        what we rendered appears on the screen """
        if self._widget and not self._destroyed:
            self._widget.SwapBuffers()

    def _SetTitle(self, title):
        """ Set the title of the figure... """
        if self._widget and not self._destroyed:
            window = self._widget.GetParent()
            if hasattr(window,'SetTitle'):
                window.SetTitle(title)
    
    def _SetPosition(self, x, y, w, h):
        """ Set the position of the widget. """
        # select widget to resize. If it
        if self._widget and not self._destroyed:
            widget = self._widget
            if isinstance(widget.GetParent(), FigureFrame):
                widget = widget.GetParent()
            # apply
            #widget.SetDimensions(x, y, w, h)
            widget.Move(x,y)
            widget.SetClientSize(w,h)
    
    def _GetPosition(self):
        """ Get the position of the widget. """
        # select widget to resize. If it
        if self._widget and not self._destroyed:
            widget = self._widget
            if isinstance(widget.GetParent(), FigureFrame):
                widget = widget.GetParent()
            # get and return
            #tmp = widget.GetRect()
            #return tmp.left, tmp.top, tmp.width, tmp.height
            size = widget.GetClientSize()
            pos = widget.GetPosition()
            return pos[0], pos[1], size[0], size[1]
        return 0, 0, 0, 0
    
    
    def _RedrawGui(self):
        if self._widget:
            self._widget.Refresh()
    
    
    def _ProcessGuiEvents(self):
        app.ProcessEvents()
    
    
    def _Close(self, widget):
        if widget is None:
            widget = self._widget
        if widget and widget.GetParent():
            try:
                widget.GetParent().Hide()
                widget.GetParent().Close()
            except wx.PyAssertionError:
                # Prevent "wxEVT_MOUSE_CAPTURE_LOST not being processed" error.
                pass


class FigureFrame(wx.Frame):
    """ Define a Frame. This is only to be able to tell whether
    the Figure object is used as a widget or as a Figure on its
    own. """
    pass


def newFigure():
    """ Create a window with a figure widget.
    """
    
    # Make sure there is a native app. Need here too, because we need to
    # create the figure frame first
    app.Create()
    
    # Create frame
    refSize = tuple( visvis.settings.figureSize )
    frame = FigureFrame(None, -1, "Figure", size=refSize)
    
    # Correct size. The given size includes the window manager's frame
    size = frame.GetClientSize()
    w = refSize[0] + (refSize[0] - size[0])
    h = refSize[1] + (refSize[1] - size[1])
    frame.SetSize((w,h))
    
    # Inser figure
    figure = Figure(frame)
    
    # Set icon
    try:
        iconFile = os.path.join(getResourceDir(), 'visvis_icon_wx.png')
        frame.SetIcon(wx.Icon(iconFile, wx.BITMAP_TYPE_PNG))
    except Exception:
        pass
    
    # Show AFTER canvas is added
    frame.Show()
    
    # Apply a draw, so that OpenGl can initialize before we will really
    # do some drawing. Otherwis textures end up showing in black.
    figure.DrawNow()
    app.ProcessEvents() # Fixes issue 43
    return figure


class VisvisEventsTimer(wx.Timer):
    def Notify(self):
        events.processVisvisEvents()




class App(events.App):
    """ App()
    
    Application class to wrap the GUI applications in a class
    with a simple interface that is the same for all backends.
    
    This is the wxPython implementation.
    
    Modifications added to check for wxPython Phoenix
    
    """
    
    def __init__(self):
        # Timer to enable timers in visvis. Should be created AFTER the app
        self._timer = None
        
        # check which version of wx is installed, set _phoenix switch
        # and select the correct _ProcessEvents
        if "phoenix" in wx.PlatformInfo:
            self._phoenix = True
            self._ProcessEvents = self._ProcessEventsPhoenix
        else:
            self._phoenix = False
            self._ProcessEvents = self._ProcessEventsOriginal
    
    def _GetNativeApp(self):
        # Get native app in save way. Taken from guisupport.py,
        # but use wx.App() class because PySimpleApp is deprecated
        app = wx.GetApp()
        if app is None:
            app = wx.App(False)
        # Store so it won't be deleted, but not on a visvis object,
        # or an application may produce error when closed
        wx.app_instance = app
        # Start timer
        if self._timer is None:
            self._timer = VisvisEventsTimer()
            self._timer.Start(10, False)
        # Return
        return app

    def _ProcessEventsOriginal(self):
        # This is the original API
        # Get app
        app = self._GetNativeApp()
        
        # Keep reference of old eventloop instance
        old = wx.EventLoop.GetActive()
        # Create new eventloop and process
        eventLoop = wx.EventLoop()
        wx.EventLoop.SetActive(eventLoop)
        while eventLoop.Pending():
            eventLoop.Dispatch()
        # Process idle
        app.ProcessIdle() # otherwise frames do not close
        # Set back the original
        wx.EventLoop.SetActive(old)
    
    def _ProcessEventsPhoenix(self):
        # this version uses the new Phoenix API
        # Get app
        app = self._GetNativeApp()
        
        # Keep reference of old eventloop instance
        old = wx.EventLoopBase.GetActive()
        # Create new eventloop and process
        eventLoop = app.GetTraits().CreateEventLoop()
        wx.EventLoopActivator(eventLoop)
        while eventLoop.Pending():
            eventLoop.Dispatch()
        # Process idle
        eventLoop.ProcessIdle() # otherwise frames do not close
        # Set back the original
        wx.EventLoopActivator(old)
    
    def _Run(self):
        app = self._GetNativeApp()
        if hasattr(app, '_in_event_loop') and app._in_event_loop:
            pass # Already in event loop
        else:
            app.MainLoop()

# Create application instance now
app = App()



# -*- coding: utf-8 -*-
# Copyright (C) 2012, Almar Klein
#
# Visvis is distributed under the terms of the (new) BSD License.
# The full license can be found in 'license.txt'.

import visvis as vv
import numpy as np

def imshow(im, clim=None, aa=2, interpolate=False, cm=None,
            axesAdjust=True, axes=None):
    """ imshow(im, clim=None, aa=2, interpolate=False, cm=CM_GRAY,
                axesAdjust=True, axes=None)
    
    Display a 2D image and returns the Texture2D object.
    
    If the image is an anisotropic array (vv.Aaray), the appropriate
    scale and translate transformations are applied.
    
    Parameters
    ----------
    im : numpy array
        The image to visualize. Can be grayscale, RGB, or RGBA.
    clim : 2-element tuple
        The color limits to scale the intensities of the image. If not given,
        the im.min() and im.max() are used (neglecting nan and inf).
    aa : 0, 1, 2 or 3
        Anti aliasing. 0 means no anti-aliasing. The highee the number, the
        better quality the anti-aliasing is (Requires a GLSL compatible
        OpenGl implementation). Default 2.
    interpolation : bool
        Use no interpolation (i.e. nearest neighbour) or linear interpolation.
    cm : Colormap
        Set the colormap to apply in case the image is grayscale.
    axesAdjust : bool
        If axesAdjust==True, this function will call axes.SetLimits(), set
        the camera type to 2D, and make axes.daspect[1] negative (i.e. flip
        the y-axis). If daspectAuto has not been set yet, it is set to False.
    axes : Axes instance
        Display the image in this axes, or the current axes if not given.
    
    Notes
    -----
    New images are positioned on z=-0.1, such that lines and points are
    visible over the image. This z-pos of textures already in the axes
    are moved backwards if new images are displayed with imshow, such that
    the new image is displayed over the older ones.
    (the changed value is `Texture2D._trafo_trans.dz`)
    
    Visvis does not use the "hold on / hold off" system. So if updating
    an image, better use Texture2D.Refresh() or call Axes.Clear() first.
    
    """
    
    # get axes
    if axes is None:
        axes = vv.gca()
    
    # Check data
    if not isinstance(im, np.ndarray):
        raise ValueError('imshow expects an image as a numpy array.')
    elif im.size==0:
        raise ValueError('imshow cannot draw arrays with zero elements.')
    #
    if im.ndim==2 or im.ndim==3 and im.shape[-1] in [1,3,4]:
        pass
    else:
        raise ValueError('imshow expects a 2D image as a numpy array.')
    
    # determine texture offset, such that lines are always on top
    # of images, and new textures are on top of older images.
    texCount = 1
    texOffset = -0.1
    for item in reversed(axes._wobjects):
        if isinstance(item, vv.Texture2D):
            texCount += 1
            item._trafo_trans.dz = texCount * texOffset
    
    # create texture
    t = vv.Texture2D(axes, im)
    t._trafo_trans.dz = texOffset
    
    # set aa and interpolation
    t.aa = aa
    t.interpolate = interpolate
    
    # set clim
    if isinstance(clim,list):
        clim = tuple(clim)
    if isinstance(clim, tuple):
        t.SetClim(clim)
    
    # set colormap
    if cm is not None:
        t.colormap = cm
    
    # set axes
    if axesAdjust:
        if axes.daspectAuto is None:
            axes.daspectAuto = False
        axes.cameraType = '2d'
        da = axes.daspect
        axes.daspect = da[0], -abs(da[1]), da[2]
        axes.SetLimits()
    
    # return
    axes.Draw()
    return t



if __name__ == '__main__':
    im = vv.imread('astronaut.png')
    im = vv.Aarray(im[:,::4,:], (1,4,1)) # Keep every 4th pixel and make them wide
    # imshow knows about anisotropic arrays!
    t = vv.imshow(im)



# -*- coding: utf-8 -*-
# Copyright (C) 2012, Almar Klein
#
# Visvis is distributed under the terms of the (new) BSD License.
# The full license can be found in 'license.txt'.

import visvis as vv

def closeAll():
    """ closeAll()
    
    Closes all figures.
    
    """
    
    for fig in [fig for fig in vv.BaseFigure._figures.values()]:
        fig.Destroy()

if __name__ == '__main__':
    import time
    # Create figures
    vv.figure()
    vv.figure()
    vv.processEvents(); vv.processEvents()
    time.sleep(1.0)
    # Close all
    vv.closeAll()



# -*- coding: utf-8 -*-
# Copyright (C) 2012, Almar Klein
#
# Visvis is distributed under the terms of the (new) BSD License.
# The full license can be found in 'license.txt'.

""" Module shaders

Defines the classes that manage shading:
  * GlslProgram: represents a GLSL program by wrapping the OpenGl stuff.
  * ShaderCode: represents the code for one GLSL program (vertex or fragment).
  * ShaderCodePart: a part of GLSL code. Multiple parts combined form a program.

"""

import OpenGL.GL as gl
import OpenGL.GL.ARB.shader_objects as gla

from visvis.core.misc import getOpenGlCapable, getExceptionInstance, basestring
import visvis as vv


# Variable for debugging / developing to display shader info logs always.
alwaysShowShaderInfoLog = False


class Shader(object):
    """ Shader()
    
    A Shader object represents the shading code for a GLSL shader. It
    provides the main interface for users to control the code, set uniforms
    etc.
    
    This class wraps three objects which can be accessed easily via
    properties:
      * program: the relatively low level interface to OpenGl.
      * vertex: represents the source code for the vertex shader.
      * fragment: represents the source code for the fragment shader.
    
    """
    
    def __init__(self):
        self._vertex = ShaderCode()
        self._fragment = ShaderCode()
        self._program = GlslProgram()
        
        self._staticUniforms = {}
        self._pendingUniforms = {}
        self._textureId = -1 # -1 means disabled
        self._texturesToDisable = []
    
    @property
    def vertex(self):
        """ Get the vertex ShaderCode object for this shader.
        """
        return self._vertex
    
    @property
    def fragment(self):
        """ Get the vertex ShaderCode object for this shader.
        """
        return self._fragment
    
    @property
    def program(self):
        """ Get the glsl program for this shading code. This is the low
        level object that compiles and enables the glsl code.
        """
        return self._program
    
    @property
    def isUsable(self):
        """ Get whether this shader is usable. If not, the hardware
        does not support glsl.
        """
        return self._program.IsUsable()
    
    @property
    def hasCode(self):
        """ Get whether this shader has any code in it.
        """
        
        # Update source code?
        if self.vertex._needRecompile:
            self.program.SetVertexShader(self.vertex.GetCode())
        if self.fragment._needRecompile:
            self.program.SetFragmentShader(self.fragment.GetCode())
        
        # Check
        return bool( self._program.HasCode() )
    
    
    def Enable(self):
        """ Enable()
        
        Enable this shader. This does a number of things:
          * Update source code of vertex and fragment shader (if necessary).
          * Compile and bind shader programs (if necessary).
          * Enables the glsl program.
          * Applies any static and pending uniforms.
          * Enables all textures that were given as uniforms.
        
        Returns False when problems occured with the compiling or linking
        of the shader code. Returns True otherwise.
        
        """
        
        # Init
        self._textureId = 0
        
        # Update source code?
        self.hasCode
        
        # Enable program (compiles if necessary)
        ok = self.program.Enable()
        if not ok:
            return False
        
        # Apply static and pending uniforms
        # Note that more uniforms can be set after this function returns.
        for name, value in self._GetStaticAndPendingUniforms().items():
            self._ApplyUniform(name, value)
        
        return True
    
    
    def EnableTextureOnly(self, *args):
        """ EnableTextureOnly(name0, name1, name2)
        
        Enable only any textures that were registered as a uniform.
        This is a convenience function that can be useful when you don't
        want to use the shader, but the fixed pipeline needs the texture
        enabled.
        
        name0 is enabled with texture-unit 0, name1 with texture-unit 1, etc.
        If the names are invalid, they are ignored.
        
        Use the normal Disable() method to disable the texture.
        
        """
        if self._textureId >= 0:
            raise RuntimeError('Shader already enabled.')
        
        # Enable
        self._textureId = 0
        
        # Get uniforms
        uniforms = self._GetStaticAndPendingUniforms()
        
        # Find textures, ignore if not present
        for i in range(len(args)):
            # Get name
            name = args[i]
            if name not in uniforms:
                continue
            # Get value
            value = uniforms[name]
            if not isinstance(value, vv.core.baseTexture.TextureObject):
                continue
            # Enable and prepare for disabling
            value.Enable(i)
            self._texturesToDisable.append(value)
    
        
    def Disable(self):
        """ Disable()
        
        Disable this shading program. This does a couple of things:
          * Disables all textures that were given as uniforms.
          * Disables the glsl program.
        
        """
        
        # Disable textures
        for value in self._texturesToDisable:
            value.Disable()
        
        # Disable program
        self.program.Disable()
        
        # Clean up
        self._textureId = -1
        self._texturesToDisable = []
    
    
    def SetUniform(self, name, value):
        """ SetUniform(name, value)
        
        Set uniform value for shader code. This is the glsl system for
        getting data into the GLSL shader.
        
        The given uniform is used during the next or current draw. This
        method is therefore to be used inside OnDraw() methods.
        
        The value can be:
            * float or int: becomes float/int in glsl
            * tuple of up to 4 elements: becomes vec/ivec, based on first value
            * vv.TextureObject: becomes sampler1D, sampler2D or sampler3D
        
        In the case of a texture, the texture is automatically enabled/disabled.
        
        """
        
        # Check
        self._CheckUniform(name, value)
        
        if self._textureId < 0:
            # Disabled; add to pending uniforms
            self._pendingUniforms[name] = value
        else:
            # Enabled; apply now
            self._ApplyUniform(name, value)
    
    
    def SetStaticUniform(self, name, value):
        """ SetStaticUniform(name, value)
        
        Set uniform value for shader code. This is the glsl system for
        getting data into the GLSL shader.
        
        The given uniform is used in all draws. This method provides
        a way to set uniforms from outside the OnDraw method, and still
        be dynamic, because the value can be a callable that returns the
        actual value.
        
        The value can be:
            * float or int: becomes float/int in glsl
            * tuple of up to 4 elements: becomes vec/ivec, based on first value
            * vv.TextureObject: becomes sampler1D, sampler2D or sampler3D
            * callable that returns one of the above.
        
        In the case of a texture, the texture is automatically enabled/disabled.
        
        """
        # Check value
        if hasattr(value, '__call__'):
            # Only check name (dummy value)
            # Value is checked during applying the uniform
            self._CheckUniform(name, 1.0)
        else:
            # Check name and value
            self._CheckUniform(name, value)
        
        # Store
        self._staticUniforms[name] = value
    
    
    def RemoveStaticUniform(self, name):
        """ RemoveStaticUniform(name)
        
        Remove a static uniform.
        
        """
        self._staticUniforms.pop(name)
    
    
    def _CheckUniform(self, name, value):
        
        # Check name
        if not isinstance(name, basestring):
            raise ValueError('Uniform name must be a string.')
        if not name:
            raise ValueError('Uniform name must be at least 1 character.')
        
        # Check value
        if not ((   isinstance(value, (float, int))    ) or
                (   isinstance(value, (list, tuple)) and len(value)<5   ) or
                (   isinstance(value, vv.core.baseTexture.TextureObject) )
                ):
            raise ValueError('Invalid value for uniform.')
    
    
    def _ApplyUniform(self, name, value):
        # No hard checks, it is assumed that these have been performed
        
        if isinstance(value, float):
            self.program.SetUniformf(name, [value])
        elif isinstance(value, int):
            self.program.SetUniformi(name, [value])
        elif isinstance(value, (list, tuple)):
            if isinstance(value[0], float):
                self.program.SetUniformf(name, value)
            elif isinstance(value[0], int):
                self.program.SetUniformi(name, value)
        elif isinstance(value, vv.core.baseTexture.TextureObject):
            #print('uniform texture', name, 'at', self._textureId)
            # Enable and register as uniform
            value.Enable(self._textureId)
            self.program.SetUniformi(name, [self._textureId])
            # Prepare for disabling, and for next texture
            self._texturesToDisable.append(value)
            self._textureId += 1
    
    
    def _GetStaticAndPendingUniforms(self):
        
        # Collect static uniforms
        uniforms = {}
        for name, value in self._staticUniforms.items():
            if hasattr(value, '__call__'):
                value = value()
                self._CheckUniform(name, value)
            uniforms[name] = value
        
        # Update with pending uniforms (override static uniforms with same name)
        uniforms.update(self._pendingUniforms)
        
        return uniforms
    
    
    def DestroyGl(self):
        self._program.DestroyGl()
    
    
    def Destroy(self):
        self._staticUniforms = {}
        self._pendingUniforms = {}
        self._texturesToDisable = []


class GlslProgram:
    """ GlslProgram()
    
    A low level class representing a GLSL (OpenGL Shading Language) program.
    It provides an easy interface for adding vertex and fragment shaders
    and setting variables used in them.
    Note: On systems that do not support shading, this class will go in
    invalid mode.
    
    """
    
    def __init__(self):
        # ids
        self._programId = 0
        self._shaderIds = []
        
        # code for the shaders
        self._fragmentCode = ''
        self._vertexCode = ''
        
        # is usable?
        self._usable = None
    
    def IsUsable(self):
        """ Returns whether the program is usable. In other words, whether
        the OpenGl driver supports GLSL.
        """
        if self._usable is None: # Not True nor False
            usable = getOpenGlCapable('2.0',
                'anti-aliasing, the clim property, colormaps and 3D rendering')
            self._usable = bool(usable) # Make very sure its not None
        return self._usable
    
    
    def HasCode(self):
        """ Returns whether the program has any code associated with it.
        If not, you shoul probably not enable it.
        """
        return bool(self._fragmentCode or self._vertexCode)
    
    
    def _IsCompiled(self):
        if not self.IsUsable():
            return False
        else:
            return ( self._programId>0 and gl.glIsProgram(self._programId) )
    
    
    def Enable(self):
        """ Start using the program. Returns True on success, False otherwise.
        """
        if not self.IsUsable():
            return True # Not a compile problem, but simply old drivers.
        
        if (self._fragmentCode or self._vertexCode) and not self._IsCompiled():
            self._CreateProgramAndShaders()
        
        if self._IsCompiled():
            gla.glUseProgramObjectARB(self._programId)
            return True
        else:
            gla.glUseProgramObjectARB(0)
            return False
    
    
    def Disable(self):
        """ Stop using the program.
        """
        if not self.IsUsable():
            return
        gla.glUseProgramObjectARB(0)
    
    
    def SetVertexShader(self, code):
        """ Create a vertex shader from code and attach to the program.
        """
        self._vertexCode = code
        self.DestroyGl()


    def SetFragmentShader(self, code):
        """ Create a fragment shader from code and attach to the program.
        """
        self._fragmentCode = code
        self.DestroyGl()
   
    
    def _CreateProgramAndShaders(self):
        # clear any old programs and shaders
        
        if self._programId < 0:
            return
        
        # clear to be sure
        self.DestroyGl()
        
        if not self._fragmentCode and not self._vertexCode:
            self._programId = -1  # don't make a shader object
            return
        
        try:
            # create program object
            self._programId = gla.glCreateProgramObjectARB()
            
            # the two shaders
            codes = [self._fragmentCode, self._vertexCode]
            types = [gl.GL_FRAGMENT_SHADER, gl.GL_VERTEX_SHADER]
            for code, type in zip(codes, types):
                # only attach shaders that do something
                if not code:
                    continue
                
                # create shader object
                myshader = gla.glCreateShaderObjectARB(type)
                self._shaderIds.append(myshader)
                
                # Set its source, pyopengl accepts the code as a string.
                gla.glShaderSourceARB(myshader, [code])
                
                # compile shading code
                gla.glCompileShaderARB(myshader)
                
                # If it went well, attach!
                if self._CheckForErrors(myshader, True, False):
                    self._programId = -1
                    return
                else:
                    gla.glAttachObjectARB(self._programId, myshader)
                
                    
            # link shader and check for errors
            gla.glLinkProgramARB(self._programId)
            if self._CheckForErrors(self._programId, False, True):
                self._programId = -1
        
        except Exception:
            self._programId = -1
            why = str(getExceptionInstance())
            print("Unable to initialize shader code. %s" % why)
    
    
    def SetUniformf(self, varname, values):
        """ SetUniformf(varname, values)
        
        A uniform is a parameter for shading code.
        Set the parameters right after enabling the program.
        values should be a list of up to four floats ( which
        are converted to float32).
        
        """
        if not self._IsCompiled():
            return
        
        # convert to floats
        values = [float(v) for v in values]
        
        # get loc, encode varname to ensure it's in byte format
        # (pyopengl will support both bytes and str as input, but at the time
        #  of writing this is not yet implemented.)
        loc = gla.glGetUniformLocationARB(self._programId, varname.encode('ascii'))
        
        try:
            # set values
            if len(values) == 1:
                gl.glUniform1f(loc, values[0])
            elif len(values) == 2:
                gl.glUniform2f(loc, values[0], values[1])
            elif len(values) == 3:
                gl.glUniform3f(loc, values[0], values[1], values[2])
            elif len(values) == 4:
                gl.glUniform4f(loc, values[0], values[1], values[2], values[3])
        except Exception:
            print('Could not set uniform in shader. "%s": %s' % (varname, repr(values)))
    
    def SetUniformi(self, varname, values):
        """ SetUniformi(varname, values)
        
        A uniform is a parameter for shading code.
        Set the parameters right after enabling the program.
        values should be a list of up to four ints ( which
        are converted to int).
        
        """
        if not self._IsCompiled():
            return
        
        # convert to floats
        values = [int(v) for v in values]
        
        # get loc, encode varname to ensure it's in byte format
        # (pyopengl will support both bytes and str as input, but at the time
        #  of writing this is not yet implemented.)
        loc = gla.glGetUniformLocationARB(self._programId, varname.encode('ascii'))
        
        # set values
        if len(values) == 1:
            gl.glUniform1i(loc, values[0])
        elif len(values) == 2:
            gl.glUniform2i(loc, values[0], values[1])
        elif len(values) == 3:
            gl.glUniform3i(loc, values[0], values[1], values[2])
        elif len(values) == 4:
            gl.glUniform4i(loc, values[0], values[1], values[2], values[3])
    
    
    def _CheckForErrors(self, glObject, checkCompile=True, checkLink=True):
        """ Check for errors in compiling and linking the given shader.
        Prints the info log if there's an error.
        Returns True if an error was found.
        """
        if checkCompile:
            ok = gl.glGetShaderiv(glObject, gl.GL_COMPILE_STATUS)
            if not ok:
                self._PrintInfoLog(glObject, "Error compiling shading code:")
                return True
            elif alwaysShowShaderInfoLog:
                self._PrintInfoLog(glObject, "Compile info log:")
        if checkLink:
            ok = gl.glGetProgramiv(glObject, gl.GL_LINK_STATUS)
            if not ok:
                self._PrintInfoLog(glObject, "Error linking shading code:")
                return True
            elif alwaysShowShaderInfoLog:
                self._PrintInfoLog(glObject, "Link info log:")
    
    
    def _PrintInfoLog(self, glObject, preamble=""):
        """ Print the info log.
        """
        log = gla.glGetInfoLogARB(glObject)
        if not isinstance(log, str):
            log = log.decode('ascii') # Make it work on Python 3
        if log:
            print(preamble + ' ' + log.rstrip())
            # If this is a renderstyle, notify user to use an alternative
            if '3D_FRAGMENT_SHADER' in self._fragmentCode:
                print('Try using a different render style.')
    
    def DestroyGl(self):
        """ DestroyGl()
        
        Clear the program.
        
        """
        # clear OpenGL stuff
        if not self.IsUsable():
            return
        if self._programId>0:
            try: gla.glDeleteObjectARB(self._programId)
            except Exception: pass
        for shaderId in self._shaderIds:
            try:  gla.glDeleteObjectARB(shaderId)
            except Exception: pass
        # reset
        self._programId = 0
        self._shaderIds[:] = []
    
    
    def __del__(self):
        " You never know when this is called."
        self.DestroyGl()


class ShaderCode(object):
    """ ShaderCode()
    
    This class represents the source code for a GLSL shader program,
    composed of multiple parts. By describing programs as a composition
    of different parts, pieces of code can be more easily re-used,
    and programs can be altered in a very flexible way. This allows
    users to for example modify existing volume renderers.
    
    Parts
    -----
    A ShaderCode instance has one or more parts which are inserted in
    one another. The first part is the base code, the next is "inserted"
    into it, and the next is inserted in the result, etc.
    
    Every part has a name, representing what it does or represents.
    Different parts that do the same thing (but in different ways)
    have the same way. This enables replacing parts (such as the render
    style for 3D textures) in a very easy and natural way. All parts
    in a ShaderCode instance must have unique names.
    
    Composition
    -----------
    Insertion of one part in the base code happens via replacement.
    The code of a ShaderCodePart consists of multiple sections, where
    each section starts with a specification of what is removed. For
    this we use two greater than characters '>>':
    ```py
    >>someValue = 3;
    someValue = 4;
    ```
    
    All code following the replacement-lines is inserted in the base
    code. Note that both the replace-code as the replacing code may
    consist of multiple lines.
    
    The matching of code is based on the stripped code, so spaces do not
    matter. The replace-code (the code behind '>>') needs to be an exact
    match, but in the source there may be other code in front and after
    the code; this works:
    ```py
    // == Base code ==
    a = b+3;
    // insert more code here
    c = a-b;
    d = a+b;
    ```
    ```py
    // == Code to insert ==
    >>more code
    float tmp = a;
    a = b;
    b = tmp;
    ```
    
    
    Standard sections
    -----------------
    For clarity we define a few standard sections: '--uniforms--',
    '--varying--', '--functions--'. When defining a function/uniform/varying,
    don't forget to include the functions/uniforms/varyings of any next parts:
    ```py
        >>--uniforms--
        uniform vec3 some_vector
        // --uniforms-- // enables the next parts to set uniforms
    ```
    
    """
    
    def __init__(self):
        
        # Init uniforms
        self._uniforms = {}
        
        # Init sub codes
        self._parts = []
        
        # Init buffer. None means it's dirty
        self._buffer = None
        
        # Flag for the program. Is set to True by the isDirty property. Is
        # only set to False by the special private property _needRecompile.
        self._needRecompileFlag = True
    
    
    @property
    def parts(self):
        """ Get a list of the parts in the shader.
        """
        return [part for part in self._parts]
    
    @property
    def partNames(self):
        """ Get a list of the part names in the shader.
        """
        return [part.name for part in self._parts]
    
    @property
    def _isDirty(self):
        """ Get whether this code is dirty and needs compiling.
        """
        
        # Check self
        dirty = self._buffer is None
        
        # Update flag for program
        if dirty:
            self._needRecompileFlag = True
        
        # Done
        return dirty
    
    
    @property
    def _needRecompile(self):
        """ Gets whether this code was changed since this method was last used.
        """
        self._isDirty # Sets _needRecompileFlag if it is really dirty
        dirty = self._needRecompileFlag
        self._needRecompileFlag = False
        return dirty
    
    
    def _IndexOfPart(self, part):
        
        parts, names = self.parts, self.partNames
        
        if isinstance(part, basestring):
            if part not in names:
                raise ValueError('part not present: %s' % part)
            else:
                return names.index(part)
        elif isinstance(part, ShaderCodePart):
            if part not in parts:
                raise ValueError('part not present: %s' % repr(part))
            else:
                return parts.index(part)
        else:
            raise ValueError('Inalid part description (must be string or ShaderCodePart).')
    
    
    def AddPart(self, part, before=None, after=None):
        """ AddPart(part, before=None, after=None)
        
        Add a ShaderCodePart instance. It can be placed before or after an
        existing part. The default places the part at the end.
        
        It is an error to add a part with of a name that is already
        present in the shader.
        
        """
        
        # Check
        if not isinstance(part, ShaderCodePart):
            raise ValueError('AddPart needs a ShaderCodePart.')
        
        # Check if already exists
        if part.name in self.partNames:
            raise ValueError('part of that name already exists: %s' % part.name)
        
        # Add
        if before and after:
            raise ValueError('Can only specify before or after; not both.')
        elif before:
            i = self._IndexOfPart(before)
            self._parts.insert(i, part)
        elif after:
            i = self._IndexOfPart(after)
            self._parts.insert(i+1, part)
        else:
            self._parts.append(part)
        
        # Signal dirty
        self._buffer = None
    
    
    def ReplacePart(self, part):
        """ ReplacePart(part)
        
        Replace an existing part, based on the name. If the given
        ShaderCodePart object would replace itself, no action is taken.
        This makes it possible to call this method on each draw without
        the risk of the code being recompiled each time.
        
        """
        
        # Check
        if not isinstance(part, ShaderCodePart):
            raise ValueError('ReplacePart needs a ShaderCodePart.')
        
        # Get index and original
        i = self._IndexOfPart(part.name)
        originalPart = self._parts[i]
        
        # Replace (or not)
        if originalPart is part:
            # No action required, so no recompilation required
            pass
        else:
            # Replace and signal dirty
            self._parts[i] = part
            self._buffer = None
    
    
    def AddOrReplace(self, part, before=None, after=None):
        """ AddOrReplace(part, before=None, after=None)
        
        Convenience function to add a part or replace it if
        a part of this name already exists.
        
        """
        if self.HasPart(part.name):
            return self.ReplacePart(part)
        else:
            return self.AddPart(part, before, after)
    
    
    def RemovePart(self, part):
        """ RemovePart(part)
        
        Remove a part. Returns True on success, and False if no part exists
        for the given name. Also accepts a part name.
        
        """
        try:
            i = self._IndexOfPart(part)
        except Exception:
            return False
        else:
            # Signal dirty
            self._buffer = None
            # Pop
            self._parts.pop(i)
            return True
    
    
    def HasPart(self, part):
        """ HasPart(part)
        
        If part is a string, checks whether a part with the given name is
        present. If part is a ShaderCodePart instance, checks whether that
        exact part is present.
        
        """
        if isinstance(part, basestring):
            return part in self.partNames
        elif isinstance(part, ShaderCodePart):
            return part in self.parts
        else:
            raise ValueError('HasPart needs name or part.')
    
    
    def Clear(self):
        """ Clear()
        
        Clear all parts.
        
        """
        self._parts = []
        
        # Signal dirty
        self._buffer = None
    
    
    def GetCode(self):
        """ GetCode()
        
        Get the total code for this shader.
        
        """
        if self._isDirty:
            self._Compile()
        return '\n'.join( [t[0] for t in self._buffer] )
    
    
    def ShowCode(self, part=None, columnLimit=79):
        """ ShowCode(part=None, columnLimit=79)
        
        Print the code with line numbers, and limiting long lines.
        Can be useful when debugging glsl code.
        
        When part is given, only display the lines that belong to the
        given part name. The 'part' argument can also be a string with
        the part name.
        
        """
        # Make sure that there is code
        self.GetCode()
        
        # Get name
        if part is None:
            name = None
        elif isinstance(part, basestring):
            name = part
        elif isinstance(part, ShaderCodePart):
            name = part.name
        else:
            raise ValueError('ShowCode needs part name or ShaderCodePart instance.')
        
        # Test name
        if name and name not in self.partNames:
            raise ValueError('Given name not present.')
        
        # Iterate through lines
        linenr = 0
        lines = []
        for line, partName in self._buffer:
            linenr += 1
            line2 = '%03i|%s' % (linenr, line)
            if len(line2) > columnLimit:
                line2 = line2[:columnLimit-3] + '...'
            if (not name) or (name == partName):
                lines.append(line2)
        print('\n'.join(lines))
    
    
    def _Compile(self):
        """ _Compile()
        
        Compile the full code by filling composing all parts/sections.
        
        """
        def dedentCode(lines):
            lines2 = []
            for line in lines:
                if isinstance(line, tuple):
                    line = line[0]
                lines2.append(line.strip())
            return '\n'.join(lines2)
        
        # Init code
        totalLines = [] # code lines: (line, partName)
        totalCode = '' # code without indentation or trailing whitespace
        
        for part in self.parts:
            
            # If totalCode is empty, simply set code
            if not totalLines:
                for line in part.code.splitlines():
                    totalLines.append( (line, part.name) )
                totalCode = dedentCode(totalLines)
                continue
            
            # Get sections in this part
            partSections, partCodes = part.CollectSections()
            
            for section, code in zip(partSections, partCodes):
                
                # Find sections
                splittedCode = totalCode.split(section)
                
                # Section present?
                if len(splittedCode) == 1:
                    #print('Warning: code section <%s> not known.' % section)
                    continue
                
                # For every occurance ...
                for i in range(len(splittedCode)-1):
                    
                    # Count where we need to insert
                    nr1 = splittedCode[i].count('\n')
                    nr2 = nr1 + 1 + section.count('\n')
                    
                    # Calculate original indentation
                    line = totalLines[nr1][0]
                    originalIndent = len(line) - len(line.lstrip())
                    
                    # Remove lines that we replace
                    for ii in reversed(range(nr1, nr2)):
                        totalLines.pop(ii)
                    
                    # Insert new lines
                    for line in reversed(code.splitlines()):
                        line = originalIndent*' ' + line
                        totalLines.insert( nr1, (line, part.name) )
                
                # Set total code
                totalCode = dedentCode(totalLines)
        
        # Done
        self._buffer = totalLines


class ShaderCodePart(object):
    """ ShaderCodePart(name, version, code)
    
    A ShaderCodePart instance represents a part of GLSL code. Multiple
    parts can be combined by replacing lines in the preceding parts.
    This makes code-reuse much easier, and allows users to for example
    implement a volume renderer without the hassle of calculating ray
    direction etc.; they can simply modify an existing renderer.
    
    Parameters
    ----------
    name : str
      The name of the part, showing what it does. Different parts that
      have the same purpose have the same name. The different render
      styles are for example implemented using parts all having the
      name 'renderstyle'.
    version : str
      What version of the part this is. For the render styles, this would
      be 'mip', 'ray', 'iso', etc. This is purely for the developer/user
      to better identify this different parts.
    code : str
      The source code. Consists of different sections. Each section starts
      with one or more lines that start with '>>'. The text behind the lines
      is what shall be replaced in the preceding code part. The following
      lines are the one that shall be inserted. Indentation does not matter.
      See core/shaders_src.py for examples.
    
    """
    def __init__(self, name, version, code):
        self._name = str(name)
        self._version = str(version)
        self._rawCode = code
        self._code = self._stripCode(code)
    
    def _stripCode(self, code):
        """ Strip the code of its minimum indentation. Also strip empty
        lines before and after the actual code.
        """
        # Strip empy trailing lines
        code = code.rstrip()
        
        # Find minimum indentation, skip first empty lines
        lines1 = []
        minIndent = 99999999
        encounteredRealLine = False
        for line1 in code.splitlines():
            line2 = line1.lstrip()
            line3 = line2.rstrip()
            if encounteredRealLine or line3:
                lines1.append(line1)
            if line3:
                encounteredRealLine = True
                indent = len(line1) - len(line2)
                minIndent = min(minIndent, indent)
        
        # Remove minimum indentation and trailing whitespace
        lines2 = []
        for line1 in lines1:
            lines2.append(line1[minIndent:].rstrip())
        return '\n'.join(lines2)
    
    @property
    def name(self):
        """ Get the name of this ShaderCodePart instance.
        """
        return self._name
    @property
    def version(self):
        """ Get the version of this ShaderCodePart instance.
        """
        return self._version
    @property
    def rawCode(self):
        """ Get the original unprocessed code of this ShaderCodePart instance.
        """
        return self._rawCode
    @property
    def code(self):
        """ Get the code of this ShaderCodePart instance.
        """
        return self._code
    def __repr__(self):
        return '<Shaderpart %s: %s>' % (self.name, self.version)
    def __str__(self):
        return self._code
    def CollectSections(self):
        """ CollectSections()
        
        Returns a two-element tuple with two list elements:
          * the replace-code for each section (can be multi-line)
          * the corresponding code to replace it with.
          
        """
        
        # Init
        sections = []
        sections_raw = []
        codes = []
        
        # Collect all section needles
        linenr = 0
        lastLinenr = -10
        for line in self.code.splitlines():
            linenr += 1
            if line.startswith('>>'):
                if linenr == lastLinenr + 1:
                    sections[-1] += '\n' + line[2:].lstrip()
                    sections_raw[-1] += '\n' + line
                else:
                    sections.append(line[2:].lstrip())
                    sections_raw.append(line)
                lastLinenr = linenr
        
        # Collect code
        code = self.code
        for section_raw in sections_raw:
            code2, _, code = code.partition(section_raw)
            codes.append(code2)
        codes.append(code)
        codes.pop(0)
        
        # Clean up codes
        codes2 = []
        for code in codes:
            code = code[1:].rstrip() + '\n'
            codes2.append(code)
        
        # Done
        return sections, codes2


# Import all shaders. Can only do the import after the ShaderCodePart is deffed
from visvis.core.shaders_2 import *  # noqa
from visvis.core.shaders_3 import *  # noqa
from visvis.core.shaders_m import *  # noqa


if __name__ == '__main__':
    
    S1 = ShaderCodePart('s1','',
    """
        // Cast ray.
        int i=0;
        while (i<n)
        {
            for (i=i; i<n; i++)
            {
                // Calculate location.
                vec3 loc = edgeLoc + float(i) * ray;
    """)
    
    S2 = ShaderCodePart('s2','',
    """
    >>int i=0;
    >>while (i<n)
    >>{
    >>for (i=i; i<n; i++)
    // reversed loop
    int i = n-1;
    while (i>0)
    {
        for (i=i; i>=0; i--)
    """)
    
    s = ShaderCode()
    s.AddPart(S1)
    s.AddPart(S2)
    print(s.GetCode())



# -*- coding: utf-8 -*-
# Copyright (C) 2012, Almar Klein
#
# Visvis is distributed under the terms of the (new) BSD License.
# The full license can be found in 'license.txt'.

import visvis as vv
import numpy as np
import OpenGL.GL as gl


def bar(data1, data2=None, bottom=None, width=0.75,
                                    axesAdjust=True, axes=None, **kwargs):
    """ bar(*args, width=0.75, axesAdjust=True, axes=None, **kwargs)
    
    Create a bar chart and returns a Bars2D instance that can be
    used to change the appearance of the bars (such as color).
    
    Usage
    -----
      * bar(H, ...) creates bars of specified height.
      * bar(X, H, ...) also supply their x-coordinates
    
    Keyword arguments
    -----------------
    width : scalar
        The width of the bars.
    color : {3-element tuple, color-char}
        The color of the bars
    colors : {3-element tuple, color-char}
        A color value for each bar.
    lc : {3-element tuple, color-char}
        The color for the bar lines (i.e. edges).
    lw : scalar
        The width of the bar lines (i.e. edges).
    axesAdjust : bool
        If True, this function will call axes.SetLimits(), and set
        the camera type to 3D. If daspectAuto has not been set yet,
        it is set to False.
    axes : Axes instance
        Display the bars in the given axes, or the current axes if not given.
    
    """
    
    # Pre check
    if True:
        try:
            if not hasattr(data1, '__len__'):
                raise ValueError
            data1 = [float(val) for val in data1]
        except Exception:
            raise ValueError('bar needs a sequence of numbers.')
    if data2 is not None:
        try:
            if not hasattr(data2, '__len__'):
                raise ValueError
            data2 = [float(val) for val in data2]
        except Exception:
            raise ValueError('bar needs a sequence of numbers.')
    
    # Parse input
    if data2 is None:
        # Only height given
        hh = data1
        xx = list(range(len(hh)))
    else:
        # All three given
        xx = data1
        hh = data2
    if bottom is not None:
        bb = [float(val) for val in bottom]
    else:
        bb = [0.0 for val in xx]
    
    # Check
    if len(hh) != len(xx):
        raise ValueError('Given arrays for bar must be of equal length.')
    
    # Get axes
    if axes is None:
        axes = vv.gca()
    
    # Create Bars instance
    bars = Bars2D(axes, xx, hh, bb, width, **kwargs)
    
    # Adjust axes
    if axesAdjust:
        if axes.daspectAuto is None:
            axes.daspectAuto = True
        axes.cameraType = '2d'
        axes.SetLimits()
    
    # Done
    axes.Draw()
    return bars
    


class Bars2D(vv.Wobject):
    """ Bars2D(parent)
    
    The Bars2D class represents a bar chart. It has a few methods to
    change the appearance of the bars.
    
    This wobject is created by the function vv.bar().
    
    """
    def __init__(self, parent, xx, hh, bb, width, **kwargs):
        vv.Wobject.__init__(self, parent)
        
        # Take care of invalid values
        # Do this here, so the invalid data points are simply not drawn
        valid = np.isfinite(xx) * np.isfinite(hh) * np.isfinite(bb)
        xx = np.array(xx)[valid]
        hh = np.array(hh)[valid]
        bb = np.array(bb)[valid]
        
        # Store data
        self._xx = xx
        self._hh = hh
        self._bb = bb
        
        # Init width
        self._width = width
        
        # Init line style
        self._colors = [(0,0,1) for i in hh]
        self._lc = (0,0,0)
        self._lw = 1
        
        # Set extra args
        for key in ['color', 'colors', 'lw', 'lc']:
            if key in kwargs:
                setattr(self, key, kwargs[key])
    
    
    def _GetLimits(self):
        
        # Get limits
        w = 0.5 * self._width
        x1, x2 = self._xx.min() - w, self._xx.max() + w
        y1, y2 = self._bb.min(), (self._bb+self._hh).max()
        z1, z2 = 0, 0
        
        # Done
        return vv.Wobject._GetLimits(self, x1, x2, y1, y2, z1, z2)
    
    
    def OnDraw(self):
        
        # Prepare
        gl.glEnableClientState(gl.GL_VERTEX_ARRAY)
        gl.glDisable(gl.GL_LINE_SMOOTH)
        
        w = self._width * 0.5
        z = 0.1
        
        for x, h, b, c in zip(self._xx, self._hh, self._bb, self._colors):
            
            # Define rectangle
            vertices = np.array([   (x-w, b, z), (x-w, b+h, z),
                                    (x+w, b+h, z), (x+w, b, z) ]).astype('float32')
            
            gl.glVertexPointerf(vertices)
            
            # Draw bg
            gl.glColor3f(c[0], c[1], c[2])
            gl.glDrawArrays(gl.GL_QUADS, 0, vertices.shape[0])
            
            # Draw outer lines
            vertices[:,2] += 0.1
            clr = self._lc
            gl.glColor3f(clr[0], clr[1], clr[2])
            gl.glLineWidth(self._lw)
            gl.glDrawArrays(gl.GL_LINE_LOOP, 0, vertices.shape[0])
        
        
        # Wrap up
        gl.glFlush()
        gl.glEnable(gl.GL_LINE_SMOOTH)
        gl.glDisableClientState(gl.GL_VERTEX_ARRAY)
        
    
    
    @vv.misc.PropWithDraw
    def lc():
        """ Get/Set the line color of the bars.
        """
        def fget(self):
            return self._lc
        def fset(self, value):
            self._lc = vv.misc.getColor(value, 'setting line color')
        return locals()
    
    @vv.misc.PropWithDraw
    def lw():
        """ Get/Set the line width of the bars.
        """
        def fget(self):
            return self._lw
        def fset(self, value):
            self._lw = float(value)
        return locals()
    
    
    @vv.misc.PropWithDraw
    def color():
        """ Get/Set the color of the bars. The getter returns the color
        of the first bar, or None if there are no bars.
        """
        def fget(self):
            if self._colors:
                return self._colors[0]
            else:
                return None
        def fset(self, value):
            c = vv.misc.getColor(value)
            self._colors = [c for h in self._hh]
        return locals()
    
    @vv.misc.PropWithDraw
    def colors():
        """ Get/Set the colors of all bars at once. The number of colors
        should match the number of bars. """
        def fget(self):
            return [c for c in self._colors]
        def fset(self, value):
            if len(value) != len(self._colors):
                raise ValueError('Number of colors should match number of bars.')
            getColor = vv.misc.getColor
            self._colors = [getColor(c) for c in value]
        return locals()


if __name__ == '__main__':
    b = vv.bar([1,2,1,5])



# -*- coding: utf-8 -*-
# Copyright (C) 2012, Almar Klein
#
# Visvis is distributed under the terms of the (new) BSD License.
# The full license can be found in 'license.txt'.

import visvis as vv

def axis(command, axes=None):
    """ axis(command, axes=None)
    
    Convenience function to set axis properties. Note that all functionality
    can also be applied via the properties of the Axis object.
    
    Parameters
    ----------
    command : string
        The setting command to apply. See below.
    axes : Axes instance
        The axes to apply the setting to. Uses the current axes by default.
    
    Possible string commands
    ------------------------
      * off: hide the axis (Axes.axis.visible = False)
      * on: show the axis (Axes.axis.visible = True)
      * equal: make a circle be displayed circular (Axes.daspectAuto = False)
      * auto: change the range for each dimension indepdently (Axes.daspectAuto = True)
      * tight: show all data (Axes.SetLimits())
      * ij: flip the y-axis (make second element of Axes.daspect negative)
      * xy: (make all elements of Axes.daspect positive)
    If you want to set an Axes' limits, use Axes.SetLimits(xlim, ylim, zlim).
    
    """
    
    # Get axes
    if axes is None:
        axes = vv.gca()
    
    if command == 'off':
        axes.axis.visible = 0
    elif command == 'on':
        axes.axis.visible = 1
    elif command == 'equal':
        axes.daspectAuto = 0
    elif command == 'auto':
        axes.daspectAuto = 1
    elif command == 'tight':
        axes.SetLimits()
    elif command == 'ij':
        da = [abs(tmp) for tmp in axes.daspect]
        axes.daspect = da[0], -abs(da[1]), da[2]
    elif command == 'xy':
        da = [abs(tmp) for tmp in axes.daspect]
        axes.daspect = da[0], abs(da[1]), da[2]
    else:
        raise ValueError('Unknown command in vv.axis().')


if __name__ == '__main__':
    l = vv.plot([1,2,3,1,4,0])
    vv.axis('off')



#!/usr/bin/env python
"""
This example illustrates embedding a visvis figure in a GTK application.
"""

import gtk
import visvis as vv

app = vv.use('gtk')

class MainWindow(gtk.Window):
    
    def __init__(self, size=(560, 420)):
        gtk.Window.__init__(self)
        
        # Create boxes and button
        hbox = gtk.HBox()
        self.add(hbox)
        vbox = gtk.VBox()
        hbox.pack_start(vbox, False, False, 0)
        button = gtk.Button('Click me')
        vbox.pack_start(button, False, False, 0)
        
        # Cteate visvis figure
        Figure = app.GetFigureClass()
        self.figure = Figure()
        hbox.pack_start(self.figure._widget, True, True, 0)
        
        # Connect signals
        button.connect('clicked', self._Plot)
        self.connect('delete_event', gtk.main_quit)
        
        # Finish
        self.set_size_request(*size)
        self.set_title("Embedding in GTK")
        self.show_all()
    
    
    def _Plot(self, *args):
        vv.figure(self.figure.nr)
        vv.clf()
        vv.plot([1,2,3,1,6])
        vv.legend(['this is a line'])


# Two ways to create the application and start the main loop
if True:
    # The visvis way. Will run in interactive mode when used in IEP or IPython.
    app.Create()
    m = MainWindow()
    app.Run()
else:
    # The native way
    m = MainWindow()
    gtk.main()



# -*- coding: utf-8 -*-
# Copyright (C) 2012, Almar Klein
#
# Visvis is distributed under the terms of the (new) BSD License.
# The full license can be found in 'license.txt'.

import visvis as vv
from visvis.utils.pypoints import Pointset
import time

class GinputHelper(object):
    """ GinputHelper()
    
    Helper class for ginput function.
    Keeps track of things.
    
    """
    
    def __init__(self):
        self.axes = None
        self.pp = None
        self.N = 0
    
    def Start(self, axes, pp, N):
        self.Unregister()
        
        if axes:
            
            # Store pointset and axes
            self.axes = axes
            self.pp = pp
            self.N = N
            
            # Register with axes
            self.axes.eventMouseDown.Bind(self.OnMouseDown)
            self.axes.eventDoubleClick.Bind(self.OnDoubleClick)
            self.axes.eventKeyDown.Bind(self.OnKeyDown)
        
    
    def Unregister(self):
        
        if self.axes:
            # Unregister axes
            self.axes.eventMouseDown.Unbind(self.OnMouseDown)
            self.axes.eventDoubleClick.Unbind(self.OnDoubleClick)
            self.axes.eventKeyDown.Unbind(self.OnKeyDown)
        
        # Remove references
        self.axes = None
        self.pp = None
        self.N = 0
    
    def OnMouseDown(self, event):
        
        # Add point
        if self.pp is not None:
            self.pp.append(event.x2d, event.y2d, 0.1)
        
        # Draw
        if self.axes:
            fig = self.axes.GetFigure()
            if fig:
                fig.DrawNow()
        
        # Is this the last point?
        if self.pp is not None and self.N and len(self.pp)==self.N:
            self.Unregister()
        
        # Accept event
        return True
    
    def OnDoubleClick(self, event):
        self.Unregister()
    
    def OnKeyDown(self, event):
        if event.key == vv.KEY_ENTER:
            self.Unregister()
            


ginputHelper = GinputHelper()


def ginput(N=0, axes=None, ms='+', **kwargs):
    """ ginput(N=0, axes=None, ms='+', **kwargs)
    
    Graphical input: select N number of points with the mouse.
    Returns a 2D pointset.
    
    Parameters
    ----------
    N : int
        The maximum number of points to capture. If N=0, will keep capturing
        until the user stops it. The capturing always stops when enter is
        pressed or the mouse is double clicked. In the latter case a final
        point is added.
    axes : Axes instance
        The axes to capture the points in, or the current axes if not given.
    ms : markerStyle
        The marker style to use for the points. See plot.
    
    Any other keyword arguments are passed to plot to show the selected
    points and the lines between them.
    
    """
    
    # Get axes
    if not axes:
        axes = vv.gca()
    
    # Get figure
    fig = axes.GetFigure()
    if not fig:
        return
    
    # Init pointset, helper, and line object
    line = vv.plot(Pointset(2), axes=axes, ms=ms, **kwargs)
    pp = line._points
    ginputHelper.Start(axes, pp, N)
    
    # Enter a loop
    while ginputHelper.axes:
        fig._ProcessGuiEvents()
        time.sleep(0.1)
    
    # Remove line object and return points
    pp = Pointset(pp[:,:2])
    line.Destroy()
    return pp


if __name__ == '__main__':
    vv.cla()
    vv.title('Selec three points.')
    print(vv.ginput(3))



# -*- coding: utf-8 -*-
# Copyright (C) 2012, Almar Klein
#
# Visvis is distributed under the terms of the (new) BSD License.
# The full license can be found in 'license.txt'.

""" The QT4 backend.


"""
import os

import visvis
from visvis import BaseFigure, events, constants
from visvis.core.misc import getResourceDir

# Load Qt libs
qtlib = visvis.backends.qtlib
if qtlib == 'pyside':
    from PySide import QtCore, QtGui, QtOpenGL
elif qtlib == 'pyqt4':
    from PyQt4 import QtCore, QtGui, QtOpenGL
elif qtlib == 'pyqt5':
    from PyQt5 import QtCore, QtWidgets, QtGui, QtOpenGL
else:
    raise ImportError('Cannot import Qt: invalid qtlib specified "%s".' %qtlib)



# NOTICE: OpenGl some problems on Ubuntu (probably due gnome).
# The drawing of the frame and background seems sometimes be done
# seperate from the opengl drawings. This means that sometimes the
# OpenGl stuff is drawn while the frame is not, which results in stuff
# hangin in "mid-air". Or while dragging the whole window, the frame
# is drawn, but in it is either rubish (qt) or gray bg (wx). When the
# frame is not visible, it is still there (you can still resize etc.)
# This is a known bug of the X Server:
# https://wiki.ubuntu.com/RedirectedDirectRendering
# A solution while the bug is not fixed is to set the visual effects off
# in System > Preferences > Appearance.
# Also note tha wx seems the less affected backend (there is a small fix
# by redrawing on a Activate event which helps a lot)



KEYMAP = {  QtCore.Qt.Key_Shift: constants.KEY_SHIFT,
            QtCore.Qt.Key_Alt: constants.KEY_ALT,
            QtCore.Qt.Key_Control: constants.KEY_CONTROL,
            QtCore.Qt.Key_Left: constants.KEY_LEFT,
            QtCore.Qt.Key_Up: constants.KEY_UP,
            QtCore.Qt.Key_Right: constants.KEY_RIGHT,
            QtCore.Qt.Key_Down: constants.KEY_DOWN,
            QtCore.Qt.Key_PageUp: constants.KEY_PAGEUP,
            QtCore.Qt.Key_PageDown: constants.KEY_PAGEDOWN,
            QtCore.Qt.Key_Enter: constants.KEY_ENTER,
            QtCore.Qt.Key_Return: constants.KEY_ENTER,
            QtCore.Qt.Key_Escape: constants.KEY_ESCAPE,
            QtCore.Qt.Key_Delete: constants.KEY_DELETE
            }

# Make uppercase letters be lowercase
for i in range(ord('A'), ord('Z')):
    KEYMAP[i] = i+32


def modifiers(event):
    """Convert the QT modifier state into a tuple of active modifier keys."""
    mod = ()
    qtmod = event.modifiers()
    if QtCore.Qt.ShiftModifier & qtmod:
        mod += constants.KEY_SHIFT,
    if QtCore.Qt.ControlModifier & qtmod:
        mod += constants.KEY_CONTROL,
    if QtCore.Qt.AltModifier & qtmod:
        mod += constants.KEY_ALT,
    return mod


class GLWidget(QtOpenGL.QGLWidget):
    """ An OpenGL widget inheriting from QtOpenGL.QGLWidget
    to pass events in the right way to the wrapping Figure class.
    """
    
    def __init__(self, figure, parent, *args):
        # Create opengl format
        glFormat = QtOpenGL.QGLFormat()
        #glFormat.setSampleBuffers(True)
        #glFormat.setSamples(4)
        #
        QtOpenGL.QGLWidget.__init__(self, glFormat, parent, *args)
        self.setAttribute(QtCore.Qt.WA_DeleteOnClose) # keep cleaned up
        self.figure = figure
        # Note that the default QGLFormat has double buffering enabled.
        
        # Set icon
        try:
            iconFile = os.path.join(getResourceDir(), 'visvis_icon_%s.png' % qtlib)
            icon = QtGui.QIcon()
            icon.addFile(iconFile, QtCore.QSize(16,16))
            self.setWindowIcon(icon)
        except Exception:
            pass
        
        # enable mouse tracking so mousemove events are always fired.
        self.setMouseTracking(True)
        
        # enable getting keyboard focus
        self.setFocusPolicy(QtCore.Qt.StrongFocus)
        self.setFocus() # make the widget have focus...
    
    
    def mousePressEvent(self, event):
        but = 0
        if event.button() == QtCore.Qt.LeftButton:
            but = 1
        elif event.button() == QtCore.Qt.RightButton:
            but = 2
        x, y = event.x(), event.y()
        self.figure._GenerateMouseEvent('down', x, y, but, modifiers(event))
    
    def mouseReleaseEvent(self, event):
        but = 0
        if event.button() == QtCore.Qt.LeftButton:
            but = 1
        elif event.button() == QtCore.Qt.RightButton:
            but = 2
        x, y = event.x(), event.y()
        self.figure._GenerateMouseEvent('up', x, y, but, modifiers(event))
    
    def mouseDoubleClickEvent(self, event):
        but = 0
        if event.button() == QtCore.Qt.LeftButton:
            but = 1
        elif event.button() == QtCore.Qt.RightButton:
            but = 2
        x, y = event.x(), event.y()
        self.figure._GenerateMouseEvent('double', x, y, but, modifiers(event))
    
    def mouseMoveEvent(self, event):
        if self.figure:
            # fire event
            x, y = event.x(), event.y()
            self.figure._GenerateMouseEvent('motion', x, y, 0, modifiers(event))
    
    def wheelEvent(self, event):
        # Get number of steps
       
        if qtlib == "pyqt5":
            numDegrees = event.angleDelta() / 8.0
        else:
            numDegrees = event.delta() / 8.0
                                    
        numSteps = numDegrees / 15.0
        
        if qtlib == "pyqt5":
            horizontal, vertical = numSteps.x(), numSteps.y()
        else:
            if event.orientation() == QtCore.Qt.Vertical:
                horizontal, vertical = 0, numSteps
            else:
                horizontal, vertical = numSteps, 0
                
        # fire event
        x, y = event.x(), event.y()
        self.figure._GenerateMouseEvent('scroll', x, y, horizontal, vertical, modifiers(event))
    
    def keyPressEvent(self, event):
        key = self._ProcessKey(event)
        text = str(event.text())
        self.figure._GenerateKeyEvent('keydown', key, text, modifiers(event))
    
    def keyReleaseEvent(self, event):
        if event.isAutoRepeat():
            return # Skip release auto repeat events
        key = self._ProcessKey(event)
        text = str(event.text())
        self.figure._GenerateKeyEvent('keyup', key, text, modifiers(event))
    
    def _ProcessKey(self,event):
        """ evaluates the keycode of qt, and transform to visvis key.
        """
        key = event.key()
        
        # special cases for shift control and alt -> map to 17 18 19
        if key in KEYMAP:
            return KEYMAP[key]
        else:
            return key
    
    def enterEvent(self, event):
        if self.figure:
            ev = self.figure.eventEnter
            ev.Set(0,0,0)
            ev.Fire()
    
    def leaveEvent(self, event):
        if self.figure:
            ev = self.figure.eventLeave
            ev.Set(0,0,0)
            ev.Fire()
    
#     def resizeEvent(self, event):
#         """ QT event when the widget is resized.
#         """
#         self.figure._OnResize()
    
    def closeEvent(self, event):
        if self.figure:
            self.figure.Destroy()
        event.accept()

    def focusInEvent(self, event):
        if self.figure:
            BaseFigure._currentNr = self.figure.nr
    
    
    def initializeGL(self):
        pass # no need
    
    def resizeGL(self, w, h):
        # This does not work if we implement resizeEvent
        self.figure._OnResize()
    
    def paintEvent(self,event):
        # Use this rather than paintGL, because the latter also swaps buffers,
        # while visvis already does that.
        # We could use self.setAutoBufferSwap(False), but it seems not to help.
        self.figure.OnDraw()
    
# This is to help draw the frame (see bug above), but I guess one should
# simply disable it's visual effects
#     def moveEvent(self, event):
#         self.update()
        
    def showEvent(self, event):
        # Force a redraw. In some applications that have stuff that takes
        # a while to draw, the content of the OpenGL widget is simply
        # fully transparent, showing through any "underlying" widgets.
        # By calling swapBuffers on "show" this is prevented.
        
        # Swapping buffers in PyQt5 will print a warning if the window is not exposed
        # "QOpenGLContext::swapBuffers() called with non-exposed window, behavior is undefined"
        if hasattr(self, 'windowHandle'):
            qwindow = self.windowHandle()
            if hasattr(qwindow, 'isExposed'):
                if not qwindow.isExposed():
                    return
        self.swapBuffers()


class Figure(BaseFigure):
    """ This is the Qt4 implementation of the figure class.
    
    A Figure represents the OpenGl context and is the root
    of the visualization tree; a Figure Wibject does not have a parent.
    
    A Figure can be created with the function vv.figure() or vv.gcf().
    """
    
    def __init__(self, parent, *args, **kwargs):
        
        # keep same documentation
        self.__doc__ = BaseFigure.__doc__
        self._widget = None
        self._widget_args = (parent, args, kwargs)
        if kwargs.get('create_widget', True):
            self.CreateWidget()
        
        # call original init AFTER we created the widget
        BaseFigure.__init__(self)
    
    def CreateWidget(self):
        """ Create the Figure's widget if necessary, and return the
        widget. """
        if self._widget is None:
            # Make sure there is a native app and the timer is started
            # (also when embedded)
            app.Create()
            
            # create widget
            updatePosition = False
            parent, args, kwargs = self._widget_args
            if 'create_widget' in kwargs:
                updatePosition = True
                del(kwargs['create_widget'])
            self._widget = GLWidget(self, parent, *args, **kwargs)
            if updatePosition:
                self.position._Changed()
        return self._widget
    
    
    def _SetCurrent(self):
        """ Make this scene the current OpenGL context.
        """
        if self._widget and not self._destroyed:
            self._widget.makeCurrent()
        
    def _SwapBuffers(self):
        """ Swap the memory and screen buffer such that
        what we rendered appears on the screen """
        if self._widget and not self._destroyed:
            self._widget.swapBuffers()
        
    def _SetTitle(self, title):
        """ Set the title of the figure. Note that this
        does not have to work if the Figure is uses as
        a widget in an application.
        """
        if self._widget and not self._destroyed:
            self._widget.setWindowTitle(title)

    def _SetPosition(self, x, y, w, h):
        """ Set the position of the widget. """
        if self._widget and not self._destroyed:
            self._widget.setGeometry(x, y, w, h)
    
    def _GetPosition(self):
        """ Get the position of the widget. """
        if self._widget and not self._destroyed:
            tmp = self._widget.geometry()
            return tmp.left(), tmp.top(), tmp.width(), tmp.height()
        return 0, 0, 0, 0
    
    def _RedrawGui(self):
        if self._widget:
            self._widget.update()
    
    def _ProcessGuiEvents(self):
        app.ProcessEvents()
#         app = QtGui.QApplication.instance()
#         app.flush()
#         app.processEvents()
    
    
    def _Close(self, widget):
        if widget is None:
            widget = self._widget
        if widget:
            widget.close()


def newFigure():
    """ function that produces a new Figure object, the widget
    in a window. """
    
    # Create figure
    fig = Figure(None)
    fig._widget.show() # In Gnome better to show before resize
    size = visvis.settings.figureSize
    fig._widget.resize(size[0],size[1])
    
    # Let OpenGl initialize and return
    fig.DrawNow()
    return fig



class App(events.App):
    """ App()
    
    Application class to wrap the GUI applications in a class
    with a simple interface that is the same for all backends.
    
    This is the Qt4 implementation.
    
    """
    
    def __init__(self):
        self._timer = None
    
    def _GetNativeApp(self):
        # Get native app in save way. Taken from guisupport.py
        app = None
        if qtlib == "pyqt5":
            app = QtWidgets.QApplication.instance()
        else:
            app = QtGui.QApplication.instance()

        if app is None:
            if qtlib == "pyqt5":
                app = QtWidgets.QApplication([''])
            else:
                app = QtGui.QApplication([''])

        # Store so it won't be deleted, but not on a visvis object,
        # or an application may produce error when closed

        if qtlib == "pyqt5":
            QtWidgets._qApp = app
        else:
            QtGui._qApp = app

        # Start timer
        if self._timer is None:
            # Instantiate timer
            self._timer = QtCore.QTimer()
            self._timer.setInterval(10)
            self._timer.setSingleShot(False)
            self._timer.timeout.connect(events.processVisvisEvents)
            self._timer.start()
        # Return
        return app
    
    def _ProcessEvents(self):
        app = self._GetNativeApp()
        app.flush()
        app.processEvents()
    
    def _Run(self):
        app = self._GetNativeApp()
        if hasattr(app, '_in_event_loop') and app._in_event_loop:
            pass # Already in event loop
        else:
            app.exec_()

# Create application instance now
app = App()



# -*- coding: utf-8 -*-
# Copyright (C) 2012, Almar Klein
#
# SSDF is distributed under the terms of the (new) BSD License.
# See http://www.opensource.org/licenses/bsd-license.php

""" Package ssdf

Simple Structured Data Format
-----------------------------
Ssdf is a simple format for stroring structured data. It supports
seven data types, two of which are container elements:
None, int, float, (Unicode) string, numpy array, list/tuple, dict/Struct.

One spec, two formats
---------------------
Ssdf is actually two formats: the original text format is human readable,
and thus enables inspecting databases and data structures using a simple
text editor. The binary format is more efficient and uses compression on
the whole file (the text format only compresses arrays). It is more suited
for storing really large databases or structures containing large arrays.
Both formats are fully compatible.

Functions of interest
---------------------
  * save    - save a struct to a file
  * saves   - serialize a struct to a string
  * saveb   - serialize a struct to bytes
  * load    - load a struct from file
  * loads   - load a struct from a string
  * loadb   - load a struct from bytes
  * update  - update a struct on file with a given struct
  * copy    - create a deep copy of a struct
  * new     - create a new empty struct
  * clear   - clear a struct, removing all elements
  * count   - count the number of elements in a struct

Notes
-----
From version 2.1, ssdf is a package (instead of a single module). The package
uses relative imports and the repository is flat, which makes it easy
to include ssdf as a subrepository in an application or library. Pyzolib
includes the ssdf package in this manner.

"""

__version__ = '2.1'

import time

# Create/import claassManager and insert two of its functions in this namespace
from .classmanager import ClassManager
register_class = ClassManager.register_class
is_compatible_class = ClassManager.is_compatible_class

# Import the rest
from . import ssdf_base
from .ssdf_base import Struct, isstruct, VirtualArray, binary_type, string_types
from . import ssdf_text, ssdf_bin



def _get_mode(filename, mode):
    """ _get_mode(filename, mode)
    
    Given filename and mode returns the mode (as 1 or 2).
    
    """
    
    # Determine mode from extension
    mode_ext = 0
    if filename.lower().endswith('.ssdf'):
        mode_ext = 1
    elif filename.lower().endswith('.bsdf'):
        mode_ext = 2
    
    # Determine given mode
    if isinstance(mode, string_types):
        mode = mode.lower()
    if mode in [1, 'text', 'string', 'str']:
        mode = 1
    elif mode in [2, 'binary', 'bin', 'bytes']:
        mode = 2
    elif mode:
        ValueError("Unknown mode given '%s'." % repr(mode))
    
    # Determine mode
    if not mode:
        mode = mode_ext
    elif mode_ext and mode_ext != mode:
        raise ValueError('Given mode does not correspond with extension.')
    if not mode:
        raise ValueError('No mode specified (and no known extension used).')
    
    # Done
    return mode


def save(filename, struct, mode=None):
    """ save(filename, struct, mode=None)
    
    Save the given struct or dict to the filesystem using the given filename.
    
    Two modes are supported: text mode stores in a human readable format,
    and binary mode stores in a more efficient (compressed) binary format.
    
    Parameters
    ----------
    filename : str
        The location in the filesystem to save the file. Files with extension
        '.ssdf' are stored in text mode, and '.bsdf' files are stored in
        binary mode. If another extension is used, the mode should be
        specified explicitly.
    struct : {Struct, dict}
        The object to save.
    mode : optional {'text', 'str', 1, 'bin', 'bytes', 2}
        This parameter can be used to explicitly specify the mode. Note
        that it is an error to use binary mode on a '.ssdf' file or text
        mode on a '.bsdf' file.
    
    """
    
    # Check
    if not (isstruct(struct) or isinstance(struct, dict)):
        raise ValueError('ssdf.save() expects the second argument to be a struct.')
    
    # Open file
    f = open(filename, 'wb')
    try:
        
        # Get mode
        mode = _get_mode(filename, mode)
        
        # Write
        if mode==1:
            writer = ssdf_text.TextSSDFWriter()
            # Write code directive and header
            header =  '# This Simple Structured Data Format (SSDF) file was '
            header += 'created from Python on %s.\n' % time.asctime()
            f.write('# -*- coding: utf-8 -*-\n'.encode('utf-8'))
            f.write(header.encode('utf-8'))
            # Write lines
            writer.write(struct, f)
        elif mode==2:
            writer = ssdf_bin.BinarySSDFWriter()
            writer.write(struct, f)
    
    finally:
        f.close()


def saves(struct):
    """ saves(struct)
    
    Serialize the given struct or dict to a (Unicode) string.
    
    Parameters
    ----------
    struct : {Struct, dict}
        The object to save.
    
    """
    # Check
    if not (isstruct(struct) or isinstance(struct, dict)):
        raise ValueError('ssdf.saves() expects a struct.')
    
    # Write
    writer = ssdf_text.TextSSDFWriter()
    return writer.write(struct)


def saveb(struct):
    """ saveb(struct)
    
    Serialize the given struct or dict to (compressed) bytes.
    
    Parameters
    ----------
    struct : {Struct, dict}
        The object to save.
    
    """
    
    # Check
    if not (isstruct(struct) or isinstance(struct, dict)):
        raise ValueError('ssdf.saveb() expects a struct.')
    
    # Write
    writer = ssdf_bin.BinarySSDFWriter()
    return writer.write(struct)


def load(filename):
    """ load(filename)
    
    Load a struct from the filesystem using the given filename.
    
    Two modes are supported: text mode stores in a human readable format,
    and binary mode stores in a more efficient (compressed) binary format.
    
    Parameters
    ----------
    filename : str
        The location in the filesystem of the file to load.
    
    """
    
    # Open file
    f = open(filename, 'rb')
    try:
    
        # Get mode
        try:
            firstfour = f.read(4).decode('utf-8')
        except Exception:
            raise ValueError('Not a valid ssdf file.')
        if firstfour == 'BSDF':
            mode = 2
        else:
            mode = 1 # This is an assumption.
        
        # Read
        f.seek(0)
        if mode==1:
            reader = ssdf_text.TextSSDFReader()
            return reader.read(f)
        elif mode==2:
            reader = ssdf_bin.BinarySSDFReader()
            return reader.read(f)
    
    finally:
        f.close()


def loadb(bb):
    """ loadb(bb)
    
    Load a struct from the given bytes.
    
    Parameters
    ----------
    bb : bytes
        A serialized struct (obtained using ssdf.saveb()).
    
    """
    # Check
    if not isinstance(bb, binary_type):
        raise ValueError('ssdf.loadb() expects bytes.')
    
    # Read
    reader = ssdf_bin.BinarySSDFReader()
    return reader.read(bb)


def loads(ss):
    """ loads(ss)
    
    Load a struct from the given string.
    
    Parameters
    ----------
    ss : (Unicode) string
        A serialized struct (obtained using ssdf.saves()).
    
    """
    # Check
    if not isinstance(ss, string_types):
        raise ValueError('ssdf.loads() expects a string.')
    
    # Read
    reader = ssdf_text.TextSSDFReader()
    return reader.read(ss)


def update(filename, struct):
    """ update(filename, struct)
    
    Update an existing ssdf file with the given struct.
    
    For every dict in the data tree, the elements are updated.
    Note that any lists occuring in both data trees are simply replaced.
    
    """
    
    # Load existing struct
    s = load(filename)
    
    # Insert stuff
    def insert(ob1, ob2):
        for name in ob2:
            if ( name in ob1 and isstruct(ob1[name]) and
                                 isstruct(ob2[name]) ):
                insert(ob1[name], ob2[name])
            else:
                ob1[name] = ob2[name]
    insert(s, struct)
    
    # Save
    save(filename, s)


def new():
    """ new()
    
    Create a new Struct object. The same as "Struct()".
    
    """
    return Struct()


def clear(struct):
    """ clear(struct)
    
    Clear all elements of the given struct object.
    
    """
    for key in [key for key in struct]:
        del(struct.__dict__[key])


def count(object):
    """ count(object):
    
    Count the number of elements in the given object.
    
    An element is defined as one of the 7 datatypes supported by ssdf
    (dict/struct, tuple/list, array, string, int, float, None).
    
    """
    
    n = 1
    if isstruct(object) or isinstance(object, dict):
        for key in object:
            val = object[key]
            n += count(val)
    elif isinstance(object, (tuple, list)):
        for val in object:
            n += count(val)
    return n


def copy(object):
    """ copy(objec)
    
    Return a deep copy the given object. The object and its children
    should be ssdf-compatible data types.
    
    Note that dicts are converted to structs and tuples to lists.
    
    """
    if isstruct(object) or isinstance(object, dict):
        newObject = Struct()
        for key in object:
            val = object[key]
            newObject[key] = copy(val)
        return newObject
    elif isinstance(object, (tuple, list)):
        return [copy(ob) for ob in object]
    elif isinstance(object, VirtualArray):
        return VirtualArray(object.shape, object.dtype, object.data)
    elif ssdf_base.np and isinstance(object, ssdf_base.np.ndarray):
        return object.copy()
    else:
        # immutable
        return object



# -*- coding: utf-8 -*-
# Copyright (C) 2015, Almar Klein
#
# Visvis is distributed under the terms of the (new) BSD License.
# The full license can be found in 'license.txt'.

""" Steps to do a new release:

Preparations:
  * Test on Windows, Linux, Mac
  * Test on a machine with OpenGl v1.1 (e.g. winXP virtual machine)
  * Make release notes
  * Update API documentation and other docs that need updating.

Test installation:
  * clear the build and dist dir (if they exist)
  * python setup.py register -r http://testpypi.python.org/pypi
  * python setup.py sdist upload -r http://testpypi.python.org/pypi
  * pip install -i http://testpypi.python.org/pypi

Define the version:
  * update __version__ in __init__.py
  * Tag the tip changeset as version x.x

Generate and upload package (preferably on Windows)
  * python setup.py register
  * python setup.py sdist upload

Announcing:
  * It can be worth waiting a day for eager users to report critical bugs
  * Announce in scipy-user, visvis mailing list, G+
  
"""

import os
from distutils.core import setup

name = 'visvis'
description = 'An object oriented approach to visualization of 1D to 4D data.'

# Get version and docstring
__version__ = None
__doc__ = ''
docStatus = 0 # Not started, in progress, done
initFile = os.path.join(os.path.dirname(__file__), '__init__.py')
for line in open(initFile).readlines():
    if (line.startswith('__version__')):
        exec(line.strip())
    elif line.startswith('"""'):
        if docStatus == 0:
            docStatus = 1
            line = line.lstrip('"')
        elif docStatus == 1:
            docStatus = 2
    if docStatus == 1:
        __doc__ += line


setup(
    name = name,
    version = __version__,
    author = 'Almar Klein',
    author_email = 'almar.klein@gmail.com',
    license = '(new) BSD',
    
    url = 'https://github.com/almarklein/visvis',
    keywords = "visualization OpenGl medical imaging 3D plotting numpy",
    description = description,
    long_description = __doc__,
    
    platforms = 'any',
    provides = ['visvis'],
    #install_requires = ['numpy', 'pyOpenGl', 'imageio'], # have imageio be auto-installed by pip?
    install_requires = ['numpy', 'pyOpenGl'],
    
    packages = ['visvis',
                'visvis.functions',
                'visvis.backends',
                'visvis.processing',
                'visvis.vvmovie',
                'visvis.vvio',
                'visvis.core',
                'visvis.wibjects',
                'visvis.wobjects',
                'visvis.text', 'visvis.text.freetype',
                'visvis.utils', 'visvis.utils.ssdf', 'visvis.utils.iso',
               ],
    package_dir = {'visvis': '.'},
    package_data = {'visvis': [ 'examples/*.py', 'visvisResources/*', 'utils/iso/*.pyx']},
    zip_safe = False,
    
    classifiers=[
          'Development Status :: 4 - Beta',
          'Intended Audience :: Science/Research',
          'Intended Audience :: Education',
          'Intended Audience :: Developers',
          'Topic :: Scientific/Engineering :: Visualization',
          'License :: OSI Approved :: BSD License',
          'Operating System :: MacOS :: MacOS X',
          'Operating System :: Microsoft :: Windows',
          'Operating System :: POSIX',
          'Programming Language :: Python',
          'Programming Language :: Python :: 2.7',
          'Programming Language :: Python :: 3.4',
          'Programming Language :: Python :: 3.5',
          'Programming Language :: Python :: 3.6',
          'Programming Language :: Python :: 3.7',
          ],
    )

# Note that the dir in package_dir must NOT be an empty string! This
# would work for distutils, but not for setuptools...
# I found this on-line doc very helpful for creating a setup script:
# http://docs.python.org/distutils/examples.html



# -*- coding: utf-8 -*-
# Copyright (C) 2012, Almar Klein
#
# Visvis is distributed under the terms of the (new) BSD License.
# The full license can be found in 'license.txt'.

import numpy as np
from visvis.wobjects.polygonalModeling import BaseMesh

def combineMeshes(meshes):
    """ combineMeshes(meshes)
    
    Given a list of mesh objects, produces a combined mesh.
    
    """
    if not meshes:
        raise ValueError('No meshes or empty meshes given')
    
    # Check mesh simularity
    vpf = 0
    hasNormals = True
    hasFaces = True
    hasValues = True
    #
    for mesh in meshes:
        if vpf == 0:
            # First mesh: init
            hasFaces = (mesh._faces is not None)
            vpf = mesh._verticesPerFace
        else:
            # Compare with first
            if mesh._verticesPerFace != vpf:
                raise ValueError('Cannot combine meshes with different verticesPerFace.')
            if (mesh._faces is not None) != hasFaces:
                raise ValueError('Cannot combine meshes with and without face data.')
        if True:
            # Compare always
            hasNormals = hasNormals and (mesh._normals is not None)
            hasValues = hasValues and (mesh._values is not None)
    
    # Combine vertices
    vertices = np.concatenate( [m._vertices for m in meshes] )
    
    # Combine faces
    faces = None
    if hasFaces:
        facesList = []
        startIndex = 0
        for mesh in meshes:
            facesList.append( mesh._faces + startIndex )
            startIndex += mesh._vertices.shape[0]
        faces = np.concatenate( facesList )
    
    # Combine normals
    normals = None
    if hasNormals:
        normals = np.concatenate( [m._normals for m in meshes] )
    
    # Combine values
    values = None
    if hasValues:
        values = np.concatenate( [m._values for m in meshes] )
    
    # Done
    return BaseMesh(vertices, faces, normals, values, vpf)



# -*- coding: utf-8 -*-
# Copyright (C) 2012, Almar Klein
#
# Visvis is distributed under the terms of the (new) BSD License.
# The full license can be found in 'license.txt'.

""" Module shaders_3

Contains the source for various shaders for the Texture3D wobject,
divided in different parts.

The names consists of different parts, seperated by underscores:
  * SH: just to indicate its a shader
  * 2F, 3F, MF, 2V, 3V, MV: the kind of program, 2D/3D texture or mesh,
    vertex or fragment shader. May be left out for some generic parts (such
    as color)
  * part name: can be BASE, STYLE, or anything else really.
  * part name type: Optional. Some parts have alternatives,
    such as the STYLE part for 3D rendering techniques. Or the SHADING
    part for meshes

"""
from visvis.core.shaders import ShaderCodePart


## 3D vertex base
# The vertex shader only has one part

SH_3V_BASE = ShaderCodePart('base', '3D-vertex-default',
"""
    // Uniforms obtained from OpenGL
    uniform vec3 shape; // The dimensions of the data
    uniform float stepRatio; // Ratio to tune the number of steps
    // --uniforms--
    
    // Varyings to pass to fragment shader
    varying vec3 ray; // The direction to cast the rays in
    varying vec3 lightDirs[1]; // The light source directions
    varying vec3 V; // The View direction
    varying vec4 vertexPosition; // The vertex position in world coordinates
    // --varyings--
    
    void main()
    {
        
        // First of all, set projected position.
        // (We need to do this because this shader replaces the original shader.)
        gl_Position = gl_ModelViewProjectionMatrix * gl_Vertex;
        
        // Store position of vertex too, so the fragment shader can calculate
        // the depth
        vertexPosition = gl_Vertex.xyzw;
        
        // Store texture coordinate (also a default thing).
        gl_TexCoord[0].xyz = gl_MultiTexCoord0.xyz;
        
        // Calculate light direction
        int nlights = 1;
        for (int i=0; i<nlights; i++)
        {
            vec4 lightPos = gl_LightSource[i].position;
            vec3 L = (lightPos * gl_ModelViewMatrix).xyz;
            if (lightPos.w==1.0)
                L -= gl_Vertex.xyz;
            float lightEnabled = float( length(lightPos) > 0.0 );
            lightDirs[i] = lightEnabled * L;
        }
        
        // Calculate view direction
        V = (vec4(0.0, 0.0, 1.0, 1.0) * gl_ModelViewMatrix).xyz;
        V = normalize(V);
        
        // Calculate ray. In projective view the result is ok at the vertices
        // but in between there can be all kind of non-linear bending of the
        // rays. To solve this, one should use a denser grid of vertex-texture
        // pairs. In textures.py, this is done by partitioning the quads.
        
        // Get location of vertex in device coordinates
        float w = max(1.0, gl_Position.w);
        vec4 refPos1 = gl_Position * gl_Position.w;
        // Calculate point right behind it. Distance depends on w-value
        // to prevent wobly artifacts at low field of views.
        float zdist = max(1.0, gl_Position.w/10.0);
        vec4 refPos2 = refPos1 + vec4(0.0, 0.0, zdist, 0.0);
        // Project back to world coordinates to calculate ray direction
        // Note: gl_ModelViewProjectionMatrixInverse does not work on Mac OSX
        vec4 p1 = gl_ModelViewMatrixInverse * gl_ProjectionMatrixInverse * refPos1;
        vec4 p2 = gl_ModelViewMatrixInverse * gl_ProjectionMatrixInverse * refPos2;
        
        ray = (p2.xyz/p2.w) - (p1.xyz/p1.w);
        
        // Normalize ray to unit length.
        ray = normalize(ray);
        
        // Make the ray represent the length of a single voxel.
        ray = ray / shape;
        ray = ray * 0.58; // 1 over root of three = 0.577
        
        // Scale ray to take smaller steps.
        ray = ray / stepRatio;
    
    }

""")


## 3D fragment base
# Base for volume rendering
# The 3D fragment shader needs calcsteps, and a type

SH_3F_BASE = ShaderCodePart('base', '3D-fragment-default',
"""
    // 3D_FRAGMENT_SHADER (used to recognize that this is a renderStyle)
    
    // Uniforms obtained from OpenGL
    uniform sampler3D texture; // The 3D texture
    uniform vec3 shape; // And its shape (as in OpenGl)
    uniform vec3 extent; // extent of the data in world coordinates
    // --uniforms--
    
    // Varyings received from fragment shader
    varying vec3 ray; // One unit step in texture coordinates
    varying vec4 vertexPosition;  // Vertex position in world coordinates
    // --varyings--
    
    // ----- Functions follow below -----
    // --functions--
    // ----- End of functions -----
    
    void main()
    {
        
        // Get current pixel location.
        vec3 edgeLoc = vec3(gl_TexCoord[0]);
        // Get number of steps
        int n = calculateSteps(edgeLoc);
        // Init iter where the depth should be
        int iter_depth = 0;
        
        // Insert more initialization here
        // --pre-loop--
        
        // Cast ray. For some reason the inner loop is not iterated the whole
        // way for large datasets. Thus this ugly hack. If you know how to do
        // it better, please let me know!
        int i=0;
        while (i<n)
        {
            for (i=i; i<n; i++)
            {
                // Calculate location.
                vec3 loc = edgeLoc + float(i) * ray;
                
                // --in-loop--
            }
        }
        
        // Insert finalization code here
        // --post-loop--
        
        
        // Discard fragment if small alpha
        if (gl_FragColor.a < 0.1)
            discard;
        
        // Set depth value
        //
        // Calculate end position in world coordinates
        vec4 position2 = vertexPosition;
        position2.xyz += ray*shape*float(iter_depth);
        // Project to device coordinates and set fragment depth
        vec4 iproj = gl_ModelViewProjectionMatrix * position2;
        iproj.z /= iproj.w;
        gl_FragDepth = (iproj.z+1.0)/2.0;
    }
    
""")


## 3D fragment calcsteps
# Calculates the amount of steps that the volume rendering should take

SH_3F_CALCSTEPS = ShaderCodePart('calcsteps', 'default',
"""
    >>--functions--
    // --functions--
    
    float d2P(vec3 p, vec3 d, vec4 P)
    {
        // calculate the distance of a point p to a plane P along direction d.
        // plane P is defined as ax + by + cz = d
        // line is defined as two points on that line
        
        // calculate nominator and denominator
        float nom = -( dot(P.rgb,p) - P.a );
        float denom =  dot(P.rgb,d);
        // determine what to return
        if (nom*denom<=0.0)
        return 9999999.0; // if negative, or ON the plane, return ~inf
        else
            return nom / denom; // return normally
    }
    
    int calculateSteps(vec3 edgeLoc)
    {
        // Given the start pos, returns a corrected version of the ray
        // and the number of steps combined in a vec4.
        
        // Check for all six planes how many rays fit from the start point.
        // Take the minimum value (not counting negative and 0).
        float smallest = 9999999.0;
        smallest = min(smallest, d2P(edgeLoc, ray, vec4(1.0, 0.0, 0.0, 0.0)));
        smallest = min(smallest, d2P(edgeLoc, ray, vec4(0.0, 1.0, 0.0, 0.0)));
        smallest = min(smallest, d2P(edgeLoc, ray, vec4(0.0, 0.0, 1.0, 0.0)));
        smallest = min(smallest, d2P(edgeLoc, ray, vec4(1.0, 0.0, 0.0, 1.0)));
        smallest = min(smallest, d2P(edgeLoc, ray, vec4(0.0, 1.0, 0.0, 1.0)));
        smallest = min(smallest, d2P(edgeLoc, ray, vec4(0.0, 0.0, 1.0, 1.0)));
        
        // round-off errors can cause the value to be very large.
        // an n of 100.000 is pretty save
        if (smallest > 9999.0)
            smallest = 0.0;
        
        // determine amount of steps
        return int( ceil(smallest) );
    }

""")


## 3D fragment STYLE MIP
# Casts a ray all the way through. Displays the highest encountered
# intensity; there is only one pixel that contributes to the final color.

SH_3F_STYLE_MIP = ShaderCodePart('renderstyle', 'mip',
"""
    >>--pre-loop--
    
    // Remember that we made sure that the total range of the data is
    // mapped between 0 and 1 (also for signed data types).
    // We track best color because resampling is inconsistent for some reason
    float val; // to store the current value
    float maxval = -99999.0; // The maximum encountered value
    float maxi = 0.0;  // Where the maximum value was encountered
    vec4 maxcolor; // The color found at the maximum value
    vec4 color1; // What we sample from the texture
    vec4 color2; // What should be displayed
    // --pre-loop--
    
    >>--in-loop--
    
    // Sample color and make value
    color1 = texture3D( texture, loc );
    --color1-to-val--
    // Bookkeeping (avoid if statements)
    float r = float(val>maxval);
    maxval = (1.0-r)*maxval + r*val;
    maxi = (1.0-r)*maxi + r*float(i);
    maxcolor = (1.0-r)*maxcolor + r*color1;
    // --in-loop--
    
    >>--post-loop--
    
    // Set depth
    iter_depth = int(maxi);
    
    // Resample color and make display-color
    //color1 = texture3D( texture, edgeLoc + float(maxi)*ray );
    color1 = maxcolor;
    // --color1-to-color2--
    gl_FragColor = color2;
    // --post-loop--
    
""")


## 3D fragment STYLE RAY
# Casts a ray all the way through. All voxels contribute (using their alpha
# value).

SH_3F_STYLE_RAY = ShaderCodePart('renderstyle', 'ray',
"""
    >>--uniforms--
    uniform float stepRatio;
    // --uniforms--
    
    >>--pre-loop--
    vec4 color1; // what we sample from the texture
    vec4 color2; // what should be displayed
    vec4 color3 = vec4(0.0, 0.0, 0.0, 0.0); // init color
    // --pre-loop--
    
    >>--in-loop--
    
    // Sample color and make display color
    color1 = texture3D( texture, loc );
    // --color1-to-color2--
    
    // Update value  by adding contribution of this voxel
    float a = color2.a * max(0.0, 1.0-color3.a) / stepRatio;
    color3.rgb += color2.rgb*a;
    color3.a += a; // color3.a counts total color contribution.
    // --in-loop--
    
    >>--post-loop--
    
    // Set depth at zero
    iter_depth = 0;
    
    // Set color
    color3.rgb /= color3.a;
    color3.a = min(1.0, color3.a);
    gl_FragColor = color3;
    // --post-loop--
    
""")


## 3D fragment STYLE EDGERAY
# Same as ray, but the alpha is scaled with the gradient magnitude, which
# makes it easier to "quickly" visualize a volume without having to play
# with the alpha channel in the ColormapEditor.

SH_3F_STYLE_EDGERAY = ShaderCodePart('renderstyle', 'edgeray',
"""
    >>--uniforms--
    uniform float stepRatio;
    // --uniforms--
    
    >>--pre-loop--
    vec4 color1; // what we sample from the texture
    vec4 color2; // what should be displayed
    vec4 color3 = vec4(0.0, 0.0, 0.0, 0.0); // init color
    vec3 step = 1.5 / shape;
    // --pre-loop--
    
    >>--in-loop--
    
    // Sample color and make display color
    color1 = texture3D( texture, loc );
    vec4 betterColor = color1;
    
    // Look in neighborhood
    // calculate normal vector from gradient
    vec3 N; // normal
    color1 = texture3D( texture, loc+vec3(-step[0],0.0,0.0) );
    color2 = texture3D( texture, loc+vec3(step[0],0.0,0.0) );
    N[0] = length(color1.rgb - color2.rgb);
    betterColor = max(max(color1, color2),betterColor);
    color1 = texture3D( texture, loc+vec3(0.0,-step[1],0.0) );
    color2 = texture3D( texture, loc+vec3(0.0,step[1],0.0) );
    N[1] = length(color1.rgb - color2.rgb);
    betterColor = max(max(color1, color2),betterColor);
    color1 = texture3D( texture, loc+vec3(0.0,0.0,-step[2]) );
    color2 = texture3D( texture, loc+vec3(0.0,0.0,step[2]) );
    N[2] = length(color1.rgb - color2.rgb);
    betterColor = max(max(color1, color2),betterColor);
    float gm = length(N); // gradient magnitude
    
    // Get "display" color
    color1 = betterColor;
    // --color1-to-color2--
    // threshold gm (this treshold can be crucial) and use to scale alpha
    gm = (gm-0.1);
    color2.a *= min(1.0, max(0.0, gm ));
    
    // Update value  by adding contribution of this voxel
    // Put bias in denominator so the first voxels dont contribute too much
    float a = color2.a * max(0.0, 1.0-color3.a) / stepRatio;
    color3.rgb += color2.rgb*a;
    color3.a += a; // color3.a counts total color contribution.
    // --in-loop--
    
    >>--post-loop--
    
    // Set depth at zero
    iter_depth = 0;
    
    // Set color
    color3.rgb /= color3.a;
    color3.a = min(1.0, color3.a);
    gl_FragColor = color3;
    // --post-loop--
    
""")


## 3D fragment STYLE ISO
# Needs Color part and litvoxel part.
# Traces a ray untill it meets a pre-defined threshold. At that location
# the lighting is calculated. When the threshold is passed X times (usually 3)
# the program exits. Final color thus corresponds to couple of voxels.

SH_3F_STYLE_ISO = ShaderCodePart('renderstyle', 'iso',
"""
    >>--uniforms--
    uniform float th; // isosurface treshold
    uniform float stepRatio;
    uniform int maxIsoSamples;
    // --uniforms--
    
    >>--pre-loop--
    vec3 step = 1.5 / shape; // Step the size of one voxel
    float val; // the value  to determine if voxel is above threshold
    vec4 color1; // temp color
    vec4 color2; // temp color
    vec4 color3 = vec4(0.0, 0.0, 0.0, 0.0); // init color
    float iter_depth_f = 0.0; // to set the depth
    int nsamples = 0;
    // --pre-loop--
    
    >>--in-loop--
    
    // Sample color and make display color
    color1 = texture3D( texture, loc );
    val = colorToVal(color1);
    
    if (val > th)
    {
        // Set color
        color2 = calculateColor(color1, loc, step);
        
        // Update value by adding contribution of this voxel
        float a = color2.a * max(0.0, 1.0-color3.a) / stepRatio;
        //float a = color2.a / ( color2.a + color3.a + 0.00001);
        color3.rgb += color2.rgb*a;
        color3.a += a; // color3.a counts total color contribution.
        
        // Set depth
        iter_depth_f = iter_depth_f + float(iter_depth_f==0.0) * float(i);
        
        // Break
        nsamples += 1;
        if (nsamples>maxIsoSamples)
        {
            i = n;
            break;
        }
    }
    // --in-loop--
    
    >>--post-loop--
    
    // Set depth
    iter_depth = int(iter_depth_f);
    
    // Set color
    color3.rgb /= color3.a;
    color3.a = min(1.0, color3.a);
    if (iter_depth_f == 0.0) // discard fragment if we encountered nothing
        color3 = vec4(0.0, 0.0, 0.0, 0.0);
    gl_FragColor = color3;
    // --post-loop--
    
""")


## 3D fragment_STYLE LITRAY
# Needs Color part and litvoxel part
# Casts a ray all the way through and calculates lighting at each step
# All contributions are combined untill alpha value saturates.
# Most pretty and most demanding for the GPU.

SH_3F_STYLE_LITRAY = ShaderCodePart('renderstyle', 'litray',
"""
    >>--uniforms--
    uniform float stepRatio;
    // --uniforms--
    
    >>--pre-loop--
    vec3 step = 1.5 / shape; // Step the size of one voxel
    float val; // the value  to determine if voxel is above threshold
    vec4 color1; // temp color
    vec4 color2; // temp color
    vec4 color3 = vec4(0.0, 0.0, 0.0, 0.0); // init color
    float iter_depth_f = 0.0; // to set the depth
    // --pre-loop--
    
    >>--in-loop--

    // Sample color and make display color
    color1 = texture3D( texture, loc );
    
    // Get color with lighting
    color2 = calculateColor(color1, loc, step);
    
    // Update value by adding contribution of this voxel
    float a = color2.a * max(0.0, 1.0-color3.a) / stepRatio;
    color3.rgb += color2.rgb*a;
    color3.a += a; // color3.a counts total color contribution.
    
    // Set depth
    iter_depth_f = iter_depth_f + float(iter_depth_f==0.0) * float(color3.a>0.5) * float(i);
    // --in-loop--
    
    >>--post-loop--
    
    // Set depth
    iter_depth = int(iter_depth_f);
    
    color3.rgb /= color3.a;
    color3.a = min(1.0, color3.a);
    gl_FragColor = color3;
    // --post-loop--
    
""")

## 3D fragment_LITVOXEL (for ISO and ISORAY styles)
# Calculates light using the binn-phong reflectance model
# Requires 'nlights' part (SH_NLIGHTS_X from shaders_m.py)

SH_3F_LITVOXEL = ShaderCodePart('litvoxel', 'default',
"""
    >>--uniforms--
    // lighting
    uniform vec4 ambient;
    uniform vec4 diffuse;
    uniform vec4 specular;
    uniform float shininess;
    // --uniforms--
    
    >>--varyings--
    varying vec3 lightDirs[1];
    varying vec3 V; // view direction
    // --varyings--
    
    
    >>--functions--
    // --functions--
    
    float colorToVal(vec4 color1)
    {
        //return color1.r;
        float val;
        // --color1-to-val--
        //val = color1.r;
        return val;
    }
    
    vec4 calculateColor(vec4 betterColor, vec3 loc, vec3 step)
    {
        // Calculate color by incorporating lighting
        vec4 color1;
        vec4 color2;
        
        // calculate normal vector from gradient
        vec3 N; // normal
        color1 = texture3D( texture, loc+vec3(-step[0],0.0,0.0) );
        color2 = texture3D( texture, loc+vec3(step[0],0.0,0.0) );
        N[0] = colorToVal(color1) - colorToVal(color2);
        betterColor = max(max(color1, color2),betterColor);
        color1 = texture3D( texture, loc+vec3(0.0,-step[1],0.0) );
        color2 = texture3D( texture, loc+vec3(0.0,step[1],0.0) );
        N[1] = colorToVal(color1) - colorToVal(color2);
        betterColor = max(max(color1, color2),betterColor);
        color1 = texture3D( texture, loc+vec3(0.0,0.0,-step[2]) );
        color2 = texture3D( texture, loc+vec3(0.0,0.0,step[2]) );
        N[2] = colorToVal(color1) - colorToVal(color2);
        betterColor = max(max(color1, color2),betterColor);
        float gm = length(N); // gradient magnitude
        N = normalize(N);
        
        // Flip normal so it points towards viewer
        float Nselect = float(dot(N,V) > 0.0);
        N = (2.0*Nselect - 1.0) * N;  // ==  Nselect * N - (1.0-Nselect)*N;
        
        // Get color of the texture (albeido)
        color1 = betterColor;
        // --color1-to-color2--
        
        // Init colors
        vec4 ambient_color = vec4(0.0, 0.0, 0.0, 0.0);
        vec4 diffuse_color = vec4(0.0, 0.0, 0.0, 0.0);
        vec4 specular_color = vec4(0.0, 0.0, 0.0, 0.0);
        vec4 final_color;
        
        int nlights = 1;
        for (int i=0; i<nlights; i++)
        {
            // Get light direction (make sure to prevent zero devision)
            vec3 L = lightDirs[i];
            float lightEnabled = float( length(L) > 0.0 );
            L = normalize(L+(1.0-lightEnabled));
            
            // Calculate lighting properties
            float lambertTerm = clamp( dot(N,L), 0.0, 1.0 );
            vec3 H = normalize(L+V); // Halfway vector
            float specularTerm = pow( max(dot(H,N),0.0), shininess);
            
            // Calculate mask
            float mask1 = lightEnabled;
            
            // Calculate colors
            ambient_color +=  mask1 * ambient * gl_LightSource[i].ambient;
            diffuse_color +=  mask1 * lambertTerm * diffuse * gl_LightSource[i].diffuse;
            specular_color += mask1 * specularTerm * specular * gl_LightSource[i].specular;
        }
        
        // Calculate final color by componing different components
        final_color = color2 * ( ambient_color + diffuse_color) + specular_color;
        final_color.a = color2.a;
        
        // Done
        return final_color;
    }
    
""")



#!/usr/bin/env python
""" This example demonstrates how one can use two of the same cameras
on the same axes.

The axes keeps a list of cameras, and only allows one camera of each type.
Therefore, in order to have two of the same cameras, we need to subclass it.

Note that in this example, there are actually 3 TwoDCamera's, as the original
is also still available (on index 2).
"""

import visvis as vv

class OurCamera1(vv.cameras.TwoDCamera):
    _NAMES = ('our1', 8)
    # a string name to be able to set cameraType
    # an int so it has an index, allowing changing the camera via a shortcut

class OurCamera2(vv.cameras.TwoDCamera):
    _NAMES = ('our2', 9)

# Draw an image
im = vv.imread('astronaut.png')
vv.imshow(im)
vv.title('Press ALT+8 for cam1 and ALT+9 for cam2')

# Get axes
a = vv.gca()

# Add cameras and select the first
a.camera = OurCamera1()
a.camera = OurCamera2()
a.cameraType = 'our1' # We can do this because we set the _NAMES attribute

# Increase zoom
a.camera.zoom *= 2

# Enter mainloop
app = vv.use()
app.Run()



#!/usr/bin/env python

""" This example illustrates picking using the simple event system.
It also shows the working of the picking in an environment.

Notice for example that the leave event is not fired when you move
the mouse beyond the boundaries of the yellow rectangle as long
as it's still inside the red rectangle. This is because the red
rectangle is a child of the yellow rectangle.
"""

import visvis as vv
app = vv.use()

# Create figure and axes
f = vv.clf()
f.description = 'Figure'
#
a = vv.gca()
a.daspectAuto = True
a.description = 'Axes'

# Create three lines
l1 = vv.plot([1,2,3,2,4], lc='b', lw=7)
l1.description = 'blue line'
#
l2 = vv.plot([4,6,5,1,3], lc='r', lw=7)
l2.description = 'red line'
#
l3 = vv.plot([2,3,1,3,2], lc='c', lw=7)
l3.description = 'cyan line'

# Create two boxes, one a child of the other
b1 = vv.Box(f)
b1.position = 0.1, 0.2, 100, 40
b1.bgcolor=(1,1,0)
b1.description = 'yellow box'
#
b2 = vv.Box(b1)
b2.position = 5, 20, 20, 50
b2.bgcolor=(1,0,1)
b2.description = 'purple box'

# Define callback functions

def picker(event):
    print("Picking %s (%i,%i - %3.2f, %3.2f)" % (event.owner.description,
            event.x, event.y, event.x2d, event.y2d))

def ignoringPicker(event):
    picker(event)
    event.Ignore() # The parent object will be picked too!

def entering(event):
    print("Entering %s" % event.owner.description)
    if hasattr(event.owner, 'lc'):
        event.owner.original_lc = event.owner.lc
        event.owner.lc = 'k'

def leaving(event):
    print("Leaving %s" % event.owner.description)
    if hasattr(event.owner, 'original_lc'):
        event.owner.lc = event.owner.original_lc

def keyCallback(event):
    print('Received key %i at %s' % (event.key, event.owner.description))


def ignoringKeyCallback(event):
    keyCallback(event)
    event.Ignore() # The parent object will receive the event too

# Subscribe to events

# Note how clicking on the red line also picks the axes,
# and clicking on the purple box also picks the yellow box.
# Note also that no mouseDown event is registered for the 3d line,
# and that clicking on it is equivalent to clicking on the axes;
# the event is propagated to the line's parent.
#
# Same goes for key events. But in addition, the figure *always*
# receives a key event.

for ob in [f, a, b1, b2, l1, l2, l3]:
    ob.eventEnter.Bind(entering)
    ob.eventLeave.Bind(leaving)
for ob in [f, a, b1, l1]:
    ob.eventMouseDown.Bind(picker)
    ob.eventKeyDown.Bind(keyCallback)
for ob in [b2, l2]:
    ob.eventMouseDown.Bind(ignoringPicker)
    ob.eventKeyDown.Bind(ignoringKeyCallback)

# Start app
app.Run()



#!/usr/bin/env python
""" This example demonstrates rendering a color volume.
We demonstrate two renderers capable of rendering color data:
the colormip and coloriso renderer.
"""

import visvis as vv
app = vv.use()

# Load volume
vol = vv.volread('stent')

# Create figure and make subplots with different renderers
vv.figure(1); vv.clf()
RS = ['mip', 'iso', 'edgeray', 'ray', 'litray']
a0 = None
tt = []
for i in range(5):
    a = vv.subplot(3,2,i+2)
    t = vv.volshow(vol)
    vv.title('Renderstyle ' + RS[i])
    t.colormap = vv.CM_HOT
    t.renderStyle = RS[i]
    t.isoThreshold = 200  # Only used in iso render style
    tt.append(t)
    if a0 is None:
        a0 = a
    else:
        a.camera = a0.camera

# Create colormap editor in first axes
cme = vv.ColormapEditor(vv.gcf(), *tt[3:])

# Run app
app.Create()
app.Run()
